/*   Main file of the galaxy simulations configuration.
     "Today" 2D simulation constants:
         GRAVIT_MODEL = STET      of course
         STET_MODE = STET_SIMPLE  because gas density is supposed low in the galaxy.
         MODE_DEBUG = RELEASE    (this is only impacting directory names).
*/
/*************************************** SIMULATION GENERAL SHAPE *******************************************/
#define NB_DIM             2
#define NB_STEP_FIRSTPHASE 1000000000   // For the 1rst phase, before cooling. DEPRECATED.
#define NB_STEP				     100000000000 // For the 2nd phase, after cooling. DEPRECATED.   

/* --------------------------------------- PERIODIC MODE ------------------------------------------------- */
#if NB_DIM == 2
  #define PERIODIC_CALCUL  0            // The galaxy or object is NOT repeated periodically.
#elif NB_DIM == 3
  #define PERIODIC_CALCUL  1            // The galaxy or object is repeated periodically
#else
#endif
  
/* --------------------------------------- GRAVITATIONAL MODELs -------------------------------------------- */
#define NEWTON            0
#define STET              1
#define TELT              2
#define GRAVIT_MODEL      STET

/* ---------------------------------------------- STET mode ------------------------------------------- */
#define STET_SIMPLE       0 
#define STET_COMPLEX      1 
#define STET_MODE         STET_SIMPLE

/********************************************* MODE DEBUG or RELEASE *****************************************/
#define DEBUG_NORM        0          // NO parallelis for correct step by step debug. HMI output in debug dir.
#define DEBUG_SOLSYST     1          // Parallelisation. HMI output in debug directories.
#define RELEASE           2          // Parallelisation. HMI output in non debug directories.
#define MODE_DEBUG        RELEASE          

/********************************************* COMPUTING *****************************************************/
#ifndef min
  #define min(a,b) (a<b ? a:b)
#endif
#ifndef max
  #define max(a,b) (a<b ? b:a)
#endif

/********************************************* GENERAL CONSTANTS *********************************************/
#define PI				        3.14159
#define MIN			          60                      // 1 min in sec.
#define HOUR              3600                    // 1 hour in sec.
#define DAY				        (24*HOUR)               // 1 day in hour.
#define MONTH				      (30*DAY)                // Roughly.
#define YEAR			        (365*DAY)               
#define MILLION_YEAR			(1000000.*YEAR)               
#define PARSEC			      3.08E16               
#define KILPARSEC		      (1000*PARSEC)
#define KILMETER		      1000
#define CM		            0.01
#define CM3		            (CM*CM*CM)
#define G_CONST           6.673E-11               // G gravitational constant.
#define MAX_LONG_DOUBLE   1.E307                  // Max value for a long double.

/* ---------------------------------------- Solar system ---------------------------------------- */
#define MASS_SUN		      2E30
#define EARTH_SUN_DIST    1.496E11                // 1.4960  10 11 m
#define V_EARTH           (2 * PI * EARTH_SUN_DIST / YEAR)

/* ----------------------------------------- Cosmology ------------------------------------------ */
#define MASS_MILKYWAY	      (5.8E11 * MASS_SUN)                   // Goes with IR_MAX = 8, fitted.
#define MASS_UGC2885	      ((100./70.) * 1.1E12 * MASS_SUN)      // Canzian et al 1993.
#define MASS_SAGITA		      (4.31E6 * MASS_SUN)   // OK so ratio of E5 which is out of nb of simul particles.
#define R_MILKYWAY          (17 * KILPARSEC)
#define DIAM_MILKYWAY       (2 * R_MILKYWAY)
#define REVOL_TIME_MILKYWAY (2E8*YEAR)            // Real galactic revolution time.
#define V_STARS_MILKYWAY    230000                // rougly 230 km/s for Milky-way stars.
#define WIDTH_MILKYWAY      (0.6 * KILPARSEC) 
       // END OF THE GALAXY : Omega = V_STARS_MILKYWAY / R_MILKYWAY =  1,2 E-2 BY-1, T = 2 pi/omega.
                           // T = 520 BY with V=200000.
                           // T = 416 BY with V=250000.   Triassic = 200 to 250 BY.
       // SUN PATH : T = (8,3 kpc/17 kpc)520 = 253 BY with V=200000.
                  // T = 203 BY with V=250000.   Triassic = 200 to 250 BY.
#define SIGMA_EXT           1.E-28                    // 1/m3  (E-28 m-3 = 1 barn)
#define RHO_C               1.E-26                          // kg/m3
#define RHO_U               (0.05 * RHO_C)
#define RHO_GAL_AROUND_SUN  0.708E-20
#define NO_GAL_AROUND_SUN   (0.67E4/CM3)              // in m-3
#define N_IGM               (1.5E-3/CM3)

extern unsigned long int StepNum;   // StepNum of the simulation.
extern int totaltime;
extern unsigned long int GetStepNum(int isReadConfig);        
extern void Error(int ErrNum);
extern double CalculTime(double total_time);
