/* HUMAN MAN INTERFACE (plotting on SciLab or whatever...)
   SciLab is used sometimes for whatching the trajectories of the particles.
   Mainly gnuplot is used, for plotting the galaxy and its characteristics on curves.
*/

#include <windows.h> 
#include <stdlib.h> 
#include <string.h> 
#include <stdio.h>
#include <math.h>
#ifdef _OPENMP
#include <omp.h>
#endif
#include "common.h"
#include "particles.h"
#include "potential.h"
#include "init.h"
#include "hmi.h"
#include "stat.h"
#include "stet.h"
#include "period.h"
#include "files_and_tools.h"

/******************** Global variables used in this module *******************/
int NbPartDisc;    // Nb of discarded particles because too far from the group.
FILE * pFileDebug;
unsigned char DebugFirstTime = 0;
long int TimePlot;
long int TimeLastPlot;
long double ComputeTimeForOneRevol;
unsigned int StepNumOld;
FILE * pFileDebugOld;

/**************** Temporary buffer for handling windows command **************/
char DirName[LGMAX_FILENAME];
char FileName[LGMAX_FILENAME];
char CmdString[LGMAX_CMDNAME];
char NameNum[LGMAX_FILENAME];
unsigned char Color[NBCOLOR] = { 'r', 'g', 'b', 'c', 'm', 'y', 'k' };

/************************* HMI module initialisation *************************/
void InitHmi()
{
  int i;

  for( i = 0; i<NB_HMI_TOOL ; i++)
  {
    if (HmiTool[i].NbMaxPartPlotted == 0)
    {
      HmiTool[i].NbMaxPartPlotted = NbPart;
      HmiTool[i].PartDelta = 1;
    }
    else
    {
      HmiTool[i].NbMaxPartPlotted = min(NbPart, HmiTool[i].NbMaxPartPlotted);
      if(HmiTool[i].NbMaxPartPlotted == 0) Error(900);
      else HmiTool[i].PartDelta = NbPart / HmiTool[i].NbMaxPartPlotted;
    }
  }

  TimePlot = TimeLastPlot = 0;
}

/************** Writing verbose information in stream's files ****************/
void PrintSimulInfoInFile(FILE *pFile, int File)
{
  char FileN[5] = "Out:";
  char *FileName = FileN;
  long double TimePerStep;
  unsigned int NbStepBetween2plot;
  unsigned long int NbStepForOneRevol;
  unsigned char CalcPot = MODE_CALCUL_POTENTIAL;
  unsigned int NbCellX;
  unsigned char Mode;
    
  if(File == 0)
  {
	  if(pFile == pFileDebugOld)
	  {
      TimePlot = GetTotalTime();
      NbStepBetween2plot = StepNum - StepNumOld;
      if(NbStepBetween2plot == 0) TimePerStep = 0;
      else TimePerStep = ((long double) (TimePlot - TimeLastPlot)) / NbStepBetween2plot; 
      NbStepForOneRevol = (unsigned long int) (REVOL_TIME / Gravit_dt); 
      ComputeTimeForOneRevol = NbStepForOneRevol * TimePerStep;
      TimeLastPlot = TimePlot;
      StepNumOld = StepNum;
    } else ;
    pFileDebugOld = pFile;
  } else ;
  if (pFile != stdout) FileName = FileSet[File].FinalNamePlusPath;
  fprintfPerso(pFile, (const char *) HmiTool[FileSet[File].FileType].CommentChar);
  fprintfPerso(pFile, " %s, et=%ld, np=%d, dty=%f, ty=%f, ct=%ld, 1revtinh=%f, StepNbRev=%d, G=%d, revt=%f, Pot=%d, NbOut=%d",
            FileName, StepNum, NbPart, Gravit_dt / YEAR, 
            total_time / YEAR, GetTotalTime(), ComputeTimeForOneRevol/HOUR,
            NbStepForOneRevol, GRAVIT_MODEL, 2*PI/(omega0*YEAR), CalcPot,
            NbPartOut);

  NbCellX = NBCELLX_GRID;
  fprintfPerso(pFile, " NbCx=%d", NbCellX);

  #if NB_DIM == 3
    fprintfPerso(pFile, " NbDim=3");
  #else
    fprintfPerso(pFile, " NbDim=2");
  #endif

  #if PERIODIC_CALCUL == 0
    fprintfPerso(pFile, " NonPer");
  #else
    fprintfPerso(pFile, " Per");
  #endif
  
  Mode = MODE_DEBUG;
  fprintfPerso(pFile, " M=%d", Mode);
  PrintParticlesInfo(pFile);
  PrintStetInfo(pFile);
  PrintStatInfo(pFile);
  fprintfPerso(pFile, "\n");
}

/************** Writing important information in stream's files **************/
void PlotPartsOneStepF(int File, unsigned long int StepNum)
{
	unsigned int i;
	int FileNum, AllDisc;
	unsigned long int StepNumInFile;
  unsigned int PartDelta;
	double pos0, pos1, pos2;
  
  if(FileSet[File].pFile == 0) return; // Case after a ReadConfig call.

	/* ---------------- The list of numerous files --------------------------- */
	FileNum = StepNum / FileSet[File].NbStepInFile;
	StepNumInFile = StepNum - FileNum * FileSet[File].NbStepInFile;
	AllDisc = 1;
	if (StepNumInFile % FileSet[File].PrintDelta == 0)
	{
		for (i = 0; i < HmiTool[FileSet[File].FileType].NbMaxPartPlotted ; i++)
		{
			if (Part[i].disc == 0) AllDisc = 0;
      PartDelta = HmiTool[FileSet[File].FileType].PartDelta;
      switch(FileSet[File].FileType)
      {
        case HMI_SCILAB:
          /* ----- SciLab ------ */
			    pos0 = Part[i*PartDelta].posold[0] / PLOT_DIST_UNIT;
			    pos1 = Part[i*PartDelta].posold[1] / PLOT_DIST_UNIT;
          fprintfPerso(FileSet[File].pFile, "x(%d, %d) = %f;\n",
            i + 1, StepNumInFile / FileSet[File].PrintDelta + 1, pos0);
          fprintfPerso(FileSet[File].pFile, "y(%d, %d) = %f;\n",
            i + 1, StepNumInFile / FileSet[File].PrintDelta + 1, pos1);
          #if (NB_DIM == 3)
            pos2 = Part[i*PartDelta].posold[2] / PLOT_DIST_UNIT;
            fprintfPerso(FileSet[File].pFile, "z(%d, %d) = %f;\n",
              i + 1, StepNumInFile / FileSet[File].PrintDelta + 1, pos2);
          #endif
          break;
        case HMI_GNUPLOT:
          /* ----- gnuplot ------ */
			    pos0 = Part[i*PartDelta].posold[0];
			    pos1 = Part[i*PartDelta].posold[1];
          pos2 = Part[i*PartDelta].posold[2];
          #if PERIODIC_CALCUL == 1
            pos0 = GetDoubleModuloPeriodic(pos0);
            pos1 = GetDoubleModuloPeriodic(pos1);
            pos2 = GetDoubleModuloPeriodic(pos2);
            if(pos0<0 || pos1<0 || pos2<0 || pos0>=LG_GRID || pos1>=LG_GRID || pos2>=LG_GRID) Error(901);
          #endif
			    pos0 /= PLOT_DIST_UNIT;
			    pos1 /= PLOT_DIST_UNIT;
          pos2 /= PLOT_DIST_UNIT;
          fprintfPerso(FileSet[File].pFile, "%f %f", pos0, pos1);
          #if (NB_DIM == 3)
            fprintfPerso(FileSet[File].pFile, " %f", pos2);
          #endif

          fprintfPerso(FileSet[File].pFile, "\n");
          break;
        case HMI_GNUPLOT_SPEEDDISP_RAD:
          /* ----- gnuplot for speed radial dispersion --------------------- */
			    pos0 = PartStat[i*PartDelta].PosNorm / PLOT_DIST_UNIT;
          fprintfPerso(FileSet[File].pFile, "%f %f\n", pos0,
                                                  PartStat[i*PartDelta].SpeedRadial);
          break;
        case HMI_GNUPLOT_SPEEDDISP_TAN:
          /* ----- gnuplot for speed dispersion tangential composant of speed*/
			    pos0 = PartStat[i*PartDelta].PosNorm / PLOT_DIST_UNIT;
          fprintfPerso(FileSet[File].pFile, "%f %f\n", pos0,
                                                  PartStat[i*PartDelta].SpeedTangent);
          break;
        case HMI_GNUPLOT_SPEEDSIGMA_RAD: 
        case HMI_GNUPLOT_SPEEDSIGMA_TAN: 
        case HMI_GNUPLOT_SPEEDMEAN_TAN:
        case HMI_GNUPLOT_MSPEEDMEAN_TAN:
        case HMI_GNUPLOT_MATD:    
        case HMI_GNUPLOT_MEANMATD:
        case HMI_GNUPLOT_LOGMEANMATD:
        case HMI_GNUPLOT_STETNBPART:
        default:
          break;
      }
		}
		if (AllDisc == 1);             // Error(35);
	}
	else;
}

void PlotPartsOneStep(unsigned long int StepNum)
{
	unsigned int FileNum, i;

	for (i = 0 ; i < NB_FILE_SET ; i++)
	{
    if(FileSet[i].NbStepInFile != 0)
    {
      FileNum = StepNum / FileSet[i].NbStepInFile;
      if(FileSet[i].NbFileMax == 0  ||  FileNum < FileSet[i].NbFileMax )
      {
        PlotPartsOneStepF(i, StepNum);
      }
      else            // Maximum nb of this type of files has been reached.
      {
        /*NOP*/       // no plot.
      }
    }
    else ;
	}
}

/******************** Execute shell for files of streams *********************/
void ExecuteShellCmdIfNeeded(int File)
{
  char FileN[5] = "Out:";
  char *FileName = FileN;

  switch(FileSet[File].FileType)
  {
    case HMI_GNUPLOT:
    case HMI_GNUPLOT_SPEEDDISP_RAD:
    case HMI_GNUPLOT_SPEEDDISP_TAN:
    case HMI_GNUPLOT_SPEEDSIGMA_RAD:
    case HMI_GNUPLOT_SPEEDSIGMA_TAN:
    case HMI_GNUPLOT_SPEEDMEAN_TAN:
    case HMI_GNUPLOT_MSPEEDMEAN_TAN:
    case HMI_GNUPLOT_MATD:    
    case HMI_GNUPLOT_MEANMATD:
    case HMI_GNUPLOT_LOGMEANMATD:
    case HMI_GNUPLOT_STETNBPART:
      strcpy(DirName, "");                   // Avoid DevC compilation bugg.                                        
      strcpy(DirName, ROOT_NAME);            /* c:\ */
      strcat(DirName, MODE_NAME);            //    s
      strcat(DirName, "\\");                 //     \    
      strcpy(CmdString, "");                 // Avoid DevC compilation bugg.                                        
      strcpy(CmdString, "cd ");                                         
      strcat(CmdString, DirName);
      systemPerso(CmdString);                     // cd c:\s\   
      strcpy(CmdString, "");                 // Avoid DevC compilation bugg.                                        
      strcpy(CmdString, "copy ");                                         
      strcat(CmdString, FileSet[File].FinalNamePlusPath);
      strcat(CmdString, " ");            
      strcat(CmdString, DirName);
      strcat(CmdString, "tmp.txt");            
      systemPerso(CmdString);                     // copy c:\s\name.txt c:\s\tmp.txt
      strcpy(CmdString, "");                 // Avoid DevC compilation bugg.                                        
      strcpy(CmdString, "gnuplot.exe ");
      strcat(CmdString, DirName);
      strcat(CmdString, (const char *) HmiTool[FileSet[File].FileType].GnuFile);  // gnuplot.exe c:\s\PlotGnuTmp3D.gnu
                             // Warning: current directory must be still available. Update it it is deleted....          
      systemPerso(CmdString);                     // Plot of tmp.txt into tmp.png with correct options.
      strcpy(CmdString, "");                 // Avoid DevC compilation bugg.                                        
      strcpy(CmdString, "del ");
      strcat(CmdString, FileSet[File].FinalNamePlusPathWithoutExt);
      strcat(CmdString, ".png");
      systemPerso(CmdString);                     // del c:\s\name.png
      strcpy(CmdString, "");                 // Avoid DevC compilation bugg.                                        
      strcpy(CmdString, "ren ");           
      strcat(CmdString, DirName);
      strcat(CmdString, "tmp.png ");           
      strcat(CmdString, FileSet[File].FinalNameWithoutExt);
      strcat(CmdString, ".png");
      systemPerso(CmdString);                     // ren c:\s\tmp.png name.png
      strcpy(CmdString, "");                 // Avoid DevC compilation bugg.                                        
      strcpy(CmdString, "del ");           
      strcat(CmdString, DirName);
      strcat(CmdString, "tmp.txt");           
      systemPerso(CmdString);                     // del tmp.txt
      strcpy(CmdString, "");                 // Avoid DevC compilation bugg.                                        
      strcpy(CmdString, "del ");                                         
      strcat(CmdString, FileSet[File].FinalNamePlusPath);
      systemPerso(CmdString);                     // del name.txt
      break;
    case HMI_SCILAB:
    default:
      break;    // Nothing.
  }
}

/*********************** OPEN CLOSE files of streams *************************/
void PlotEndFileF(int File)
{
	unsigned int i;
	unsigned long int LastNum;

    if(FileSet[File].pFile == 0) return; // Case after a ReadConfig call.

	LastNum = FileSet[File].NbPrintInFile;

	/* ----- SciLab ------ */
	for (i = 0; i < HmiTool[FileSet[File].FileType].NbMaxPartPlotted ; i++)
	{
      switch(FileSet[File].FileType)
      {
        case HMI_SCILAB:
          /* ----- SciLab ------ */
          #if (NB_DIM == 3)
            fprintfPerso(FileSet[File].pFile, "plot(x(%d, [1:1:%d]), y(%d, [1:1:%d]), z(%d, [1:1:%d]), '%c-', 'markersize', 4);\n",
                      i + 1, LastNum, i + 1, LastNum, i + 1, LastNum, Color[i%NBCOLOR]);
            fprintfPerso(FileSet[File].pFile, "set(gca(),\"auto_clear\",\"off\"); \n");
            fprintfPerso(FileSet[File].pFile, "plot(x(%d, %d), y(%d, %d), z(%d, %d), '.%c', 'markersize', 4);\n",
                      i + 1, LastNum, i + 1, LastNum, i + 1, LastNum, Color[i%NBCOLOR]);
            fprintfPerso(FileSet[File].pFile, "set(gca(),\"auto_clear\",\"off\");\n");
          #else
            fprintfPerso(FileSet[File].pFile, "plot(x(%d, [1:1:%d]),  y(%d, [1:1:%d]), '%c-', 'markersize', 4);\n",
                      i + 1, LastNum, i + 1, LastNum, Color[i%NBCOLOR]);
            fprintfPerso(FileSet[File].pFile, "set(gca(),\"auto_clear\",\"off\"); \n");
            fprintfPerso(FileSet[File].pFile, "plot(x(%d, %d),  y(%d, %d), '.%c', 'markersize', 4);\n",
                      i + 1, LastNum, i + 1, LastNum, Color[i%NBCOLOR]);
            fprintfPerso(FileSet[File].pFile, "set(gca(),\"auto_clear\",\"off\");\n");
          #endif
          break;
        case HMI_GNUPLOT_SPEEDSIGMA_RAD:
          PlotRadSigmaFile(FileSet[File].pFile);
          break;
        case HMI_GNUPLOT_SPEEDSIGMA_TAN:
          PlotTanSigmaFile(FileSet[File].pFile);
          break;
        case HMI_GNUPLOT_SPEEDMEAN_TAN:
          PlotTanMeanFile(FileSet[File].pFile);      
          break;
        case HMI_GNUPLOT_MSPEEDMEAN_TAN:
          PlotMeanTanMeanFile(FileSet[File].pFile);
          break;
        case HMI_GNUPLOT:
        case HMI_GNUPLOT_SPEEDDISP_RAD:
        case HMI_GNUPLOT_SPEEDDISP_TAN:
          /* ----- gnuplot ------ */
          // Nothing.
          break;
        case HMI_GNUPLOT_MATD:    
          PlotSurfacicMatterDensityFile(FileSet[File].pFile);
          break;
        case HMI_GNUPLOT_MEANMATD:    
          PlotMeanSurfacicMatterDensityFile(FileSet[File].pFile);
          break;
        case HMI_GNUPLOT_LOGMEANMATD:    
          PlotLogSurfacicMatterDensityFile(FileSet[File].pFile);
          break;
        case HMI_GNUPLOT_STETNBPART:    
          PlotStetNbPartFile(FileSet[File].pFile);
          break;
        default:
          break;
      }
	}

  switch(FileSet[File].FileType)
  {
    case HMI_SCILAB:
          /* ----- SciLab ------ */
          #if (NB_DIM == 3)
            #if MODE_DEBUG != DEBUG_SOLSYST  &&   MODE_DEBUG != DEBUG_NORM
              fprintfPerso(FileSet[File].pFile, "LgMax = %f;\n", RMAX_PLOT/PLOT_DIST_UNIT);
              fprintfPerso(FileSet[File].pFile, "a = get(\"current_axes\"); a.data_bounds = [-LgMax, -LgMax, -LgMax ; LgMax, LgMax, LgMax]; a.box = \"off\";\n");
            #endif
          #else
            #if MODE_DEBUG != DEBUG_SOLSYST  &&   MODE_DEBUG != DEBUG_NORM
              fprintfPerso(FileSet[File].pFile, "LgMax = %f;\n", RMAX_PLOT);
              fprintfPerso(FileSet[File].pFile, "a = get(\"current_axes\"); a.data_bounds = [-LgMax, -LgMax; LgMax, LgMax]; a.box = \"off\";\n");
            #endif
          #endif
          break;
    case HMI_GNUPLOT:
    case HMI_GNUPLOT_SPEEDDISP_RAD:
    case HMI_GNUPLOT_SPEEDDISP_TAN:
    case HMI_GNUPLOT_MATD:    
    case HMI_GNUPLOT_MEANMATD:
    case HMI_GNUPLOT_LOGMEANMATD:
    case HMI_GNUPLOT_STETNBPART:
    default:
          /* ----- gnuplot ------ */
          // Nothing.
          break;
  }

  PrintSimulInfoInFile(FileSet[File].pFile, File);
  fclosePerso(FileSet[File].pFile);
  ExecuteShellCmdIfNeeded(File);
}

void PlotEndFile()
{
	int i;
	for (i = 0; i < NB_FILE_SET ; i++)
	{
    if(FileSet[i].NbStepInFile != 0)
    {
      PlotEndFileF(i);
    }
    else ;
	}
}

void PlotOpenCloseFileIfNeededF(int File, unsigned long int StepNum)
{
	unsigned long int NumRest, FileNum;

	NumRest = StepNum % FileSet[File].NbStepInFile;
	FileNum = StepNum / FileSet[File].NbStepInFile;

	if (NumRest == 0)
	{
		if (StepNum == 0)
    {
      /*NOP*/     // First file to be opened; no previous file. This first file has been opened yet.
    }
		else
		{
      if(FileSet[File].NbFileMax == 0  ||  FileNum <= FileSet[File].NbFileMax )
      {
        if(FileSet[File].pFile != 0) PlotEndFileF(File); 
      }
      else ;
		}

    if(FileSet[File].NbFileMax == 0  ||  FileNum < FileSet[File].NbFileMax )
    {
      strcpy(FileName, "");                 // Avoid DevC compilation bugg.                                        
      strcpy(FileName, (const char *) HmiTool[FileSet[File].FileType].SoftName);
      strcat(FileName, FileSet[File].SetName);
      sprintf(NameNum, "%d", FileNum);
      strcat(FileName, NameNum);
      strcpy(FileSet[File].FinalNameWithoutExt, "");   // Avoid DevC compil bugg.                                        
      strcpy(FileSet[File].FinalNameWithoutExt, FileName);
      strcpy(FileName, "");
      strcpy(FileName, ROOT_NAME);
      strcat(FileName, MODE_NAME);
      strcat(FileName, "\\");
      strcat(FileName, FileSet[File].FinalNameWithoutExt);
      strcpy(FileSet[File].FinalNamePlusPathWithoutExt, "");
      strcpy(FileSet[File].FinalNamePlusPathWithoutExt, FileName);
      strcat(FileName, ".txt");
      strcpy(FileSet[File].FinalNamePlusPath, "");
      strcpy(FileSet[File].FinalNamePlusPath, FileName);
      FileSet[File].pFile = fopenPerso(FileName, "w");
      if (FileSet[File].pFile == NULL) Error(902);
        else;                                               

      /* ----- SciLab stack size ------ */
      if(FileSet[File].FileType == HMI_SCILAB) fprintfPerso(FileSet[File].pFile, "stacksize(268435454);\n");
      else ; // Nothing if GalCenter.
    
      PrintSimulInfoInFile(FileSet[File].pFile, File);

    }
    else ;    // NOP
	}
}

void PlotOpenCloseFileIfNeeded(unsigned long int StepNum)
{
  int i;
  for (i = 0 ; i < NB_FILE_SET ; i++)
  {
    if(FileSet[i].NbStepInFile != 0)
    {
      PlotOpenCloseFileIfNeededF(i, StepNum);
    }
    else ;
  }
}

/*********************** MISCELLANEOUS FUNCTIONS *****************************/
long int GetTotalTime()
{
	SYSTEMTIME timetest;
	GetLocalTime(&timetest);
	return(timetest.wDay * 24 * 3600 + timetest.wHour * 3600 + timetest.wMinute * 60 + timetest.wSecond);
}

void PlotInitDiscardedBeforeLoop()
{
	NbPartDisc = 0;
}

void PlotDetectDiscarded(int PartNum)
{
	double DistFrom0;

	DistFrom0 = Part[PartNum].pos[0] * Part[PartNum].pos[0];
	DistFrom0 += Part[PartNum].pos[1] * Part[PartNum].pos[1];
  #if (NB_DIM == 3)
  	DistFrom0 += Part[PartNum].pos[2] * Part[PartNum].pos[2];
  #endif
	DistFrom0 = sqrt(DistFrom0);
	if (DistFrom0 > DISCARD_DIST)
	{
		Part[PartNum].disc = 1;
		NbPartDisc++;
	}
	else
	{
		Part[PartNum].disc = 0;
	}
}

void InitFileName()
{
  strcpy(FileName, "");
  strcpy(FileName, ROOT_NAME);
  strcat(FileName, MODE_NAME);
  strcat(FileName, "\\");
  strcat(FileName, CONFIG_FILE_NAME);
}

/*********************** HANDLING CONFIGURATION FILES ************************/
void CopyConfigFile()
{
  strcpy(DirName, "");         
  strcpy(DirName, ROOT_NAME);            /* c:\ */
  strcat(DirName, MODE_NAME);            //    s
  strcat(DirName, "\\");                 //     \    
  strcpy(CmdString, "");                 // Avoid DevC compilation bugg.                                       
  strcpy(CmdString, "cd ");                                         
  strcat(CmdString, DirName);
  systemPerso(CmdString);                     // cd c:\s\  
  strcpy(CmdString, "");                                         
  strcpy(CmdString, "copy ");                                         
  strcat(CmdString, DirName);
  strcat(CmdString, CONFIG_FILE_NAME);
  strcat(CmdString, " ");
  strcat(CmdString, DirName);
  strcat(CmdString, CONFIG_FILE_NAME_WITHOUTEXT);
  sprintf(NameNum, "%d", StepNum);
  strcat(CmdString, NameNum);
  strcat(CmdString, CONFIG_FILE_EXT);
  systemPerso(CmdString);
}

void PlotConfig()
{
  FILE * pFile;
  unsigned int i;
  
  if(StepNum % WAIT_PLOT_CONF != 0) return;

  InitFileName();
  pFile = fopenConfig(FileName, "w");
  if (pFile == NULL) Error(903);
  else;
  
  fprintfConfigInt(pFile, "%d", StepNum);
  fprintfConfigChar(pFile, "%c", SEPARATION_CHAR);
  fprintfConfig(pFile, " ");
  fprintfConfigInt(pFile, "%d", NbPart);
  fprintfConfigChar(pFile, "%c", SEPARATION_CHAR);
  fprintfConfig(pFile, " ");
  #if (GRAVIT_MODEL==STET)
    fprintfConfigInt(pFile, "%d", ((int) IntAddedNbPart));
  #endif
  fprintfConfig(pFile, "\n");

  for( i=0 ; i< NbPart ; i++)
  {
    fprintfConfigDouble(pFile, "%e", Part[i].pos[0]);
    fprintfConfigChar(pFile, "%c", SEPARATION_CHAR);
    fprintfConfig(pFile, " ");
    fprintfConfigDouble(pFile, "%e", Part[i].pos[1]);
    fprintfConfigChar(pFile, "%c", SEPARATION_CHAR);
    fprintfConfig(pFile, " ");
    fprintfConfigDouble(pFile, "%e", Part[i].posold[0]);
    fprintfConfigChar(pFile, "%c", SEPARATION_CHAR);
    fprintfConfig(pFile, " ");
    fprintfConfigDouble(pFile, "%e", Part[i].posold[1]);
    fprintfConfig( pFile, "\n" );
  }
  fprintfConfig( pFile, "\n" );
	fcloseConfig(pFile);
  CopyConfigFile();    // Saving Config.txt into ConfigSTEPNUM.txt
}

unsigned char GetWord(FILE * pFile)
{
  unsigned char c;
  int i;
  
  c = fgetc(pFile);
  for( i=0 ; i<LGMAX_CMDNAME-1 && c != '\n'  &&  c != SEPARATION_CHAR  &&  c != Eof  &&  c != EOF ; i++)
  {
    CmdString[i] = c;
    c = fgetc(pFile);
  }
  CmdString[i] = '\0';
  return(c);
}

int ReadConfig()
{
  FILE * pFile;
  unsigned int i, tmp;
    
  InitFileName();
  pFile = fopenConfig(FileName, "r");
  if (pFile == NULL) Error(904);
  else;

  if(GetWord(pFile) == Eof) { fcloseConfig(pFile); return(0); }
  else sscanf(CmdString, "%d", &StepNum);
  if(GetWord(pFile) == Eof) { fcloseConfig(pFile); return(2); }
  else sscanf(CmdString, "%d", &NbPart);
  if(GetWord(pFile) == Eof) { fcloseConfig(pFile); return(2); }
  #if (GRAVIT_MODEL==STET)
    else
    {
      sscanf(CmdString, "%d", &tmp);
      IntAddedNbPart = (double) tmp;
    }
  #else
    else ;
  #endif

  for( i=0 ; i<NbPart ; i++)
  {
    if(GetWord(pFile) == Eof) { fcloseConfig(pFile); return(2); }
    else Part[i].pos[0] = atof(CmdString);
    if(GetWord(pFile) == Eof) { fcloseConfig(pFile); return(2); } 
    else Part[i].pos[1] = atof(CmdString);
    if(GetWord(pFile) == Eof) { fcloseConfig(pFile); return(2); }
    else Part[i].posold[0] = atof(CmdString);
    if(GetWord(pFile) == Eof) { fcloseConfig(pFile); return(2); }
    else Part[i].posold[1] = atof(CmdString);
  }
	fcloseConfig(pFile);
  return(1);                      // OK
}

int TryToReadConfig()        
{
  int Cr = ReadConfig();     // But read this in a file if it exists, not empty.
  if(Cr == 2) Error(905);   // Corrupted Config.txt file content
  else;
  return(Cr);
}

/************************** LOW LEVEL FILE HANDLING **************************/
#if (SILENT_MODE == 1)
FILE * fopenConfig(char *FileName, char *type)
{
  FILE * Cr = fopen(FileName,type);
  if(Cr != 0) return(Cr);                
  else return((FILE *) -1);
}

int	fcloseConfig(FILE *stream)
{
  if(stream != (FILE *) -1) return( fclose(stream) );
  else return(1);
}

int fprintfConfig(FILE *stream, const char *format)
{
  if(stream != (FILE *) -1) return( fprintf(stream, format) );
  else return(0);
}
int fprintfConfigDouble(FILE *stream, const char *format, double p)
{
  if(stream != (FILE *) -1) return( fprintf(stream, format, p) );
  else return(0);
}
int fprintfConfigChar(FILE *stream, const char *format, char p)
{
  if(stream != (FILE *) -1) return( fprintf(stream, format, p) );
  else return(0);
}
int fprintfConfigInt(FILE *stream, const char *format, int p)
{
  if(stream != (FILE *) -1) return( fprintf(stream, format, p) );
  else return(0);
}
#endif

FILE * fopenSilent(char *FileName, char *type) { return((FILE *)1); }
int	fcloseSilent(FILE *stream) { return(1); }
int fprintfSilent(FILE *stream, const char *format, ...) { return(0); }
int systemSilent(char *format) { return(0); }
