/*   Human Machine Interface
*/

/************************************************* SILENT MODE ***********************************************/
#define SILENT_MODE      0           // 1 : Avoid stopping on error, and avoid any printf(). Else 0
#if (SILENT_MODE == 0)
  #define fopenPerso     fopen
  #define fclosePerso    fclose
  #define fprintfPerso   fprintf
  #define systemPerso    system
  #define fopenConfig    fopen
  #define fcloseConfig   fclose
  #define fprintfConfig  fprintf
  #define fprintfConfigDouble fprintf
  #define fprintfConfigChar   fprintf
  #define fprintfConfigInt    fprintf
#elif (SILENT_MODE == 1)
  #define fopenPerso     fopenSilent
  #define fclosePerso    fcloseSilent
  #define fprintfPerso   fprintfSilent
  #define systemPerso    systemSilent
#else
#endif

/************************************************* GENERAL *************************************************/
#define PRINT_ECHO                  1                    // For console print echoes.

/* ------------------------------- Plotting units ------------------------------------ */
#if (NB_DIM == 2)
  #if MODE_DEBUG == DEBUG_SOLSYST
    #define PLOT_DIST_UNIT              EARTH_SUN_DIST
  #else
    #define PLOT_DIST_UNIT              KILPARSEC
  #endif
#elif (NB_DIM == 3)
    #define PLOT_DIST_UNIT              EARTH_SUN_DIST
#endif
#define PLOT_SPEED_UNIT                 KILMETER

/* ------------------------------- Directory for files -------------------------------- */
#if MODE_DEBUG == DEBUG_SOLSYST || MODE_DEBUG == DEBUG_NORM
  #define ROOT_NAME                 "c:\\Debug\\"        // Beginning the path name of the files.
#else
  #define ROOT_NAME                 "c:\\"               // Beginning the path name of the files.
#endif

/****************************************** Discarding particles mechanism *******************************/
#define DISCARD_DIST	  (8 * R_MAX)             // Above this dist from (0,0) the part is calc but not ploted.
#define RMAX_PLOT       (1.1*R_MAX)
#define XMAX_PLOT       (1.1*R_MAX)
#define YMAX_PLOT       (1.1*R_MAX)
extern int NbPartDisc;              // Nb of discarded particles because too far from the group.

/****************************************** SciLab line colors *******************************************/
#define NBCOLOR			    7
extern unsigned char    Color[NBCOLOR];

/****************************************** Saving config *******************************************/
#if (NBCELLX_GRID==128)
  #define WAIT_PLOT_CONF  10000 
#else
  #define WAIT_PLOT_CONF  1000        // Number of steps between each saving config.
#endif

#define CONFIG_FILE_EXT               ".txt"
#define CONFIG_FILE_NAME_WITHOUTEXT   "Config"
#define CONFIG_FILE_NAME              "Config.txt"
#define SEPARATION_CHAR ','
#define Eof             255

/***************************************** MACROS and FUNCTIONS *****************************************/
extern void InitHmi();
extern void PlotPartsOneStep(unsigned long int ind);
extern void PlotOpenCloseFileIfNeeded(unsigned long int StepNum);
extern void PlotEndFile();
extern long int GetTotalTime();
extern void PlotInitDiscardedBeforeLoop();
extern void PlotDetectDiscarded(int ParticleNum);
extern void PrintSimulInfoInFile(FILE *pFile, int File);
extern void PlotDebugOpen();
extern void PlotDebug(int i);
extern void PlotDebugClose();
extern void PlotConfig();
extern int ReadConfig();
extern int TryToReadConfig();
#if (SILENT_MODE == 1)
  extern FILE * fopenConfig(char *FileName, char *type);
  extern int	fcloseConfig(FILE *stream);
  extern int fprintfConfig(FILE *stream, const char *format);
  extern int fprintfConfigDouble(FILE *stream, const char *format, double p);
  extern int fprintfConfigChar(FILE *stream, const char *format, char p);
  extern int fprintfConfigInt(FILE *stream, const char *format, int p);
#endif
extern FILE * fopenSilent(char *FileName, char *type);
extern int	fcloseSilent(FILE *stream);
extern int fprintfSilent(FILE *stream, const char *format, ...);
extern int systemSilent(char *format);
