/*   INITIALISATION OF THE GROUP OF PARTICLES (initialisation of the "galaxy").
     Only InitPartDiscFrom0() is used for 2D "HOHL like" simulations.
*/

#include <stdlib.h> 
#include <string.h> 
#include <stdio.h>
#include <math.h>
#ifdef _OPENMP          
#include <omp.h>
#endif
#include "common.h"
#include "particles.h"
#include "potential.h"
#include "stet.h"
#include "hmi.h"
#include "init.h"
#include "init3d.h"

double omega0;   // Here for debug.

double NormalGauss()
{
	double x1, x2, w, y1, y2;

	do {
		x1 = 2.0 * rand() / RAND_MAX - 1.0;
		x2 = 2.0 * rand() / RAND_MAX - 1.0;
		w = x1 * x1 + x2 * x2;
	} while (w >= 1.0);

	w = sqrt((-2.0 * log(w)) / w);
	y1 = x1 * w;
	y2 = x2 * w;
	return(y1);
}

void RazSolSyst()               // Fixing Sun(s) to 0,0 same location for avoiding Gravit_dt hysteresis.
{
  #if INIT_TYPE == INIT_SOLSYST || INIT_TYPE == INIT_3D_SOLSYST || INIT_TYPE == INIT_3D_SOLSYST_OBL
	  unsigned int i;
    #define OLDVERSION
    #ifdef OLDVERSION
	    for(i = 1; i < NbPart; i++)
    #else
      for(i = 2; i < NbPart; i++)
    #endif
	  {
      Part[i].acc[0] = Part[i].acc[1] = Part[i].acc[2] = 0;
      Part[i].posold[0] = Part[i].posold[1] = Part[i].posold[2] = 0;
      Part[i].pos[0] = Part[i].pos[1] = Part[i].pos[2] = 0;
    }
  #endif                           // For avoiding Gravit_dt "hysteresis".
}

void InitSystSol()                         // Init of solar system for testing this prog.
{
  #define OLDVERSION

  #ifdef OLDVERSION
    NbPart = NBPART_SYSTSOL;
	  if (NbPart > NBMAX_PART) Error(700);
	  PartMass = MASS_SUN / (NbPart - 1);
	  Part[0].posold[0] = EARTH_SUN_DIST;
	  Part[0].pos[0] = EARTH_SUN_DIST;
	  Part[0].posold[1] = - V_EARTH * Gravit_dt;
  #else
	  NbPart = NBPART_SYSTSOL;
	  if(NbPart > NBMAX_PART) Error(701);
	  PartMass = MASS_SUN / (NbPart - 2);
	  Part[0].posold[0] = 1.1 * EARTH_SUN_DIST;
	  Part[0].pos[0] =    1.1 * EARTH_SUN_DIST;
	  Part[0].posold[1] = - V_EARTH * Gravit_dt * 0.8;
	  Part[1].posold[0] = 0.9 * EARTH_SUN_DIST;
	  Part[1].pos[0] =    0.9 * EARTH_SUN_DIST;
	  Part[1].posold[1] = - V_EARTH * Gravit_dt * 1.2; 
  #endif

	RazSolSyst();
}

void InitPartLine()                    // Init of a aligned particles with different speeds.
{
	unsigned int i;
	NbPart = NBPART_LINE;
	if (NbPart > NBMAX_PART) Error(702);
	for (i = 1; i < NbPart; i++)
	{
		Part[i].posold[0] = PART_DIST * (i+0.2);
//		Part[i].posold[1] = - V0 * Gravit_dt * sqrt(i + 1);
		Part[i].posold[1] = 0;
		Part[i].pos[0] =    PART_DIST * (i+0.2);
		Part[i].pos[1] =    V0 * Gravit_dt * sqrt(i+1);
	}
}

void InitPartSquare()                        // Init of a square with constant matter density.
{
	int i, j, ind;
	double x0, y0, dist0x, dist0y, dist0_2;

	NbPart = NBPART_SQUARE;
	if (NbPart > NBMAX_PART) Error(703);
	x0 = y0 = CORR * (NBPART2 - 1) * PART_DIST / 2;       // Middle of the group.
	for (i = 0; i < NBPART2; i++)
	{
		for (j = 0; j < NBPART2; j++)
		{
			ind = i * NBPART2 + j;
			Part[ind].acc[0] = 0;
			Part[ind].acc[1] = 0;
			Part[ind].acc[2] = 0;
			Part[ind].posold[0] = PART_DIST * i;
			Part[ind].posold[1] = PART_DIST * j;
			Part[ind].posold[2] = 0;
			dist0x = Part[ind].posold[0] - x0;
			dist0y = Part[ind].posold[1] - y0;
			dist0_2 = dist0x*dist0x + dist0y*dist0y;
			if (dist0_2 == 0) dist0_2 = 1;            // That is a short distance also in m.
			Part[ind].pos[0] = PART_DIST * i - V0 * Gravit_dt * (dist0y / dist0_2)*(NBPART2 - 1)*PART_DIST / 2;
			Part[ind].pos[1] = PART_DIST * j + V0 * Gravit_dt * (dist0x / dist0_2)*(NBPART2 - 1)*PART_DIST / 2;
			Part[ind].pos[2] = 0;
		}
	}
}

double GetSpeedInit(int PartNum)
{
	double dist0, dist1, dist_from_0, acc0, acc1, accel;

	dist0 = Part[PartNum].pos[0];
	dist1 = Part[PartNum].pos[1];
	dist_from_0 = sqrt(dist0*dist0 + dist1*dist1);
	acc0 = Part[PartNum].acc[0];
	acc1 = Part[PartNum].acc[1];
	accel = sqrt(acc0*acc0 + acc1*acc1);
	return(sqrt(accel * dist_from_0));                  // Because v/r = acc.
}

void InitPartDisc(int InitType, double speed)         // Init of a disk.
{
	int ialpha, ir, ind;
	double r, dr, r0, alpha0, speed_r;

	/* ------------------------- A bunch of particles --------------------------- */
	r0 = R_MAX / 2;
	ind = 0;                                 // Indice of the particle in the array.
	for (ir = IR_MAX - 1, r = R_MAX; ir >= 0 && r>0 ; ir--, r -= dr)
	{
		switch (InitType)                              // Direct for lisibility (no preprocessor ifdef)
		{
		case INIT_DISC_EXP:
			dr = PART_DIST * exp(r / r0);    // Exponential matter density.
			dr = max(dr, DR_MIN);       // Because if density too high it decrease enormously dt.
			break;                      // Exponential density in the galaxy.
		case INIT_DISC_CST:             // Case of constant matter density.
		case INIT_DISC_CST_EQ:          // Constant matter density in equilibrium (balanced).
			dr = PART_DIST;
			break;
		}
		alpha0 = 2 * PI * ((double)rand()) / RAND_MAX;
		for (ialpha = 0; (ialpha + 1) * dr < 2 * PI*r; ialpha++)
		{
			double alpha;                               // Angle in radian in polar coord.
			alpha = alpha0 + ialpha * dr / r;           // Random turn of ring for sparing heterogeneities.
			Part[ind].posold[0] = r * cos(alpha);
			Part[ind].posold[1] = r * sin(alpha);
			switch (InitType)                              // Direct for lisibility (no preprocessor ifdef)
			{
			case INIT_DISC_CST_EQ:
				speed_r = GetSpeedInit(ind);
				break;
			default:
				speed_r = speed;
				break;
			}
			Part[ind].pos[0] = Part[ind].posold[0] - speed_r * sin(alpha) * Gravit_dt;  // Velocities
			Part[ind].pos[1] = Part[ind].posold[1] + speed_r * cos(alpha) * Gravit_dt;  // in galaxy.
			Part[ind].acc[0] = 0;
			Part[ind].acc[1] = 0;
			Part[ind].disc = 0;
			if (++ind >= NBMAX_PART) Error(704);       // Dont get out of the array!
			else continue;
		}                                             // End of loop on alpha.
	}
	switch (InitType)                                 // Direct for lisibility (no preprocessor ifdef)
	{
	case INIT_DISC_CST:                 // Case of constant matter density.
	case INIT_DISC_CST_EQ:      
		/* --------------------- And a Black hole in the middle of it !? -------------------------- */
		// But today it gets same mass as other partic ! Just another particle in middle of the group.
		Part[ind].acc[0] = 0; Part[ind].acc[1] = 0; Part[ind].posold[0] = 0; Part[ind].posold[1] = 0;
		Part[ind].pos[0] = 0; Part[ind].pos[1] = 0; Part[ind].disc = 0;
		NbPart = ind + 1;
		break;
	case INIT_DISC_EXP:
		NbPart = ind;
		break;                      // Exponential density in the galaxy.
	}
	if (NbPart == 0) Error(705);
	else
	{
      PartMass = MASS_MILKYWAY_CORR / NbPart;
	}
}

void InitPartDiscFrom0()                        // Init of a disk beginning from the center.
{
	int ialpha, ind;
	double r, dr, alpha0, speed_r; 
  double Mu, dr0, rand0, rand1;
  double MaxSpeed;

	/* ------------------------- A particle in the middle --------------------------- */
	/* Thats for r=0 :           */
	ind = 0;                                 // Indice of the particle in the array.
	Part[ind].pos[0] = Part[ind].posold[0] = Part[ind].pos[1] = Part[ind].posold[1] = 0;
	Part[ind].acc[0] = Part[ind].acc[1] = 0;
	Part[ind].disc = 0;

	/* ------------------------- A bunch of particles --------------------------- */
	ind = 1;
	Mu = MU0;
	omega0 = OMEGA0;                            // Caution there is a sqrt() there.
	dr0 = PART_DIST;
	for (r = dr0; r < R_MAX; r+=dr)
	{
		dr = sqrt(MU0 / Mu) * dr0;
		alpha0 = 2 * PI * ((double)rand()) / RAND_MAX;
		for (ialpha = 0; (ialpha + 1) * dr < 2 * PI*r; ialpha++)
		{
			double alpha, Sigma;                        // Angle in radian in polar coord.

			/* ----- Calculating initial positions of particles ------------------------ */
			alpha = alpha0 + ialpha * dr / r;           // Random turn of ring for sparing heterogeneities.
			Part[ind].posold[0] = r * cos(alpha);
			Part[ind].posold[1] = r * sin(alpha);

			/* ----- Calculating mean velocities ------------------------------------------ */
			speed_r = omega0 * r;                       // Constant angular velocity.
			Part[ind].pos[0] = Part[ind].posold[0] + speed_r * sin(alpha) * Gravit_dt;  // Velocities
			Part[ind].pos[1] = Part[ind].posold[1] - speed_r * cos(alpha) * Gravit_dt;  // in galaxy.

			/* ----- Adding Gaussian dispersion of velocities ------- */
			Sigma = SPEED_GAUSS_SIGMA * R_MAX * omega0 * sqrt(1 - r*r / (R_MAX*R_MAX));   // Refer to HOHL's article.
			rand0 = Sigma * NormalGauss();
			rand1 = Sigma * NormalGauss();       // Independant call from the first one.
			Part[ind].pos[0] += -rand0 * sin(alpha) * Gravit_dt;
			Part[ind].pos[1] += rand0 * cos(alpha) * Gravit_dt;;
			Part[ind].pos[0] += rand1 * cos(alpha) * Gravit_dt;
			Part[ind].pos[1] += rand1 * sin(alpha) * Gravit_dt;;

			/* ----- Those are calculated later on ------ */
			Part[ind].acc[0] = 0;
			Part[ind].acc[1] = 0;
			Part[ind].disc = 0;
			if (++ind >= NBMAX_PART) Error(706);       // Dont get out of the array!
			else continue;
		}                                             // End of loop on alpha.
		Mu = MU0 * sqrt(1 - r*r / (R_MAX*R_MAX));     // Matter density of the next ring.
	}                                                 // end of loop on r.
	NbPart = ind;
	if (NbPart == 0) Error(707);
	else
	{
       PartMass = TOTAL_MASS / NbPart;
	}
 	MaxSpeed = omega0 * R_MAX;
  Gravit_dt = DT; 
}

void InitPartZero()                    // Init of all field with 0.
{
	int i;
	for(i = 0; i < NBMAX_PART; i++)
	{
		Part[i].pos[0] = Part[i].pos[1] = Part[i].pos[2] = 0;
    Part[i].posold[0] = Part[i].posold[1] = Part[i].posold[2] = 0;
		Part[i].acc[0] = Part[i].acc[1] = Part[i].acc[2] = 0;
	}
}

/* ---------------- Cooling the actual galaxy ------------------------------ */
/*
       Inversion of matrix for transforming speeds in tangential-radial base:

1) Calculating components of speed in radial-tangential base:
     rad = s0 r0 + s1 r1
     tan = r0 s1 - r1 s0

2) Transforming speeds in radial-tangential base for cooling the galaxy:
     rad = 0
     if tan < 0 then tan = 0

3) Inverting base transform matrix:
       r0 s0 + r1 s1 = rad
     - r1 s0 + r0 s1 = tan

     D = r0 +r1 = RadialVector = 1
     |s0|  = |r0 -r1|  |rad|
     |s1|    |r1  r0|  |tan|

4) Calculating components of transformed speed in normal base:
     s0 = (1/D) (r0 rad - r1 tan)
     s1 = (1/D) (r1 rad + r0 tan)

5) Calculating new positions (pos) using old ones (posold) and previously cooled speeds.
*/
void Cooling()
{
	unsigned int i;
  double Speed[2], Pos[2], PosNorm, RadialVector[2], PosOrigin[2];
  double SpeedRadial, SpeedTangent, DeltaPos[2];
  
  /* --- 0) Retrieving speed in normal base-------------------------------------- */
  VectorZero(PosOrigin);
  for(i = 0; i < NbPart; i++)
  {
    VectorAdd(Part[i].pos, PosOrigin, PosOrigin);         // PosOrigin += pos
  }
  VectorDiv(PosOrigin, NbPart, PosOrigin);                // PosOrigin = barycentric of Particles posit.

  for(i = 0; i < NbPart; i++)
  {
    VectorSub(Part[i].pos, PosOrigin, Pos);               // Pos = pos(i) - origin.
    VectorNorm(Pos, &PosNorm);                            // PosNorm = ||Pos||

    if(MODE_DEBUG == DEBUG_SOLSYST)
    {
      /* NOP */    // Solar system should not be used there anyhow.
    }  
    else  
    {
      if(PosNorm == 0) Error(708);                        // Hardly probable.
      else
      {
        VectorDiv(Pos, PosNorm, RadialVector);            // RadialVector = pos/||pos||
      }
    }
    VectorSub(Part[i].pos, Part[i].posold, DeltaPos);     // DeltaPos = pos - posold.
    VectorDiv(DeltaPos, Gravit_dt, Speed);                // Speed = DeltaPos / dt.

    /* --- 1) Calculating components of speed in radial-tangential base ---------------- */
    SpeedRadial  = Speed[0] * RadialVector[0] + Speed[1] * RadialVector[1];   // Scalar product v.ur
    SpeedTangent = RadialVector[0] * Speed[1] - RadialVector[1] * Speed[0];   // Vectorial product ur^v.

    /* --- 2) Transforming speeds in radial-tangential base for cooling the galaxy: --- */
    SpeedRadial = 0;
    if(SpeedTangent > 0) SpeedTangent = - SpeedTangent;  // Forbid reverse sense.
    else ; 
    
    /* --- 3) Inverting base transform matrix: --- */
    /* --- 4) Calculating components of transformed speed in normal base: -------------- */
    Speed[0] = RadialVector[0] * SpeedRadial - RadialVector[1] * SpeedTangent;
    Speed[1] = RadialVector[1] * SpeedRadial + RadialVector[0] * SpeedTangent;

    /* --- 5) Calculating new positions ------------------------------------------------ */
    VectorMul(Speed, Gravit_dt, DeltaPos);                // DeltaPos = speed x dt.
    VectorAdd(Part[i].posold, DeltaPos, Part[i].pos);     // pos = posold + DeltaPos.
  }
}

void InitPart(int InitType)                        // Init of a disk with exponential exp(r/r0) density.
{
	Gravit_dt = DT;
	PartMass = PART_MASS;                          // Default value.
	InitPartZero();

	switch (InitType)
	{
	case INIT_LINE:
		InitPartLine();
		break;
	case INIT_SQUARE:
		InitPartSquare();
		break;
	case INIT_DISC_EXP:
	case INIT_DISC_CST:
		InitPartDisc(InitType, V0);
		break;
	case INIT_DISC_CST_EQ:
		InitPartDisc(INIT_DISC_CST, 0);
    DensityCalculate();          
		GravitPotentialCalculate();  
		GravitFieldCalculate();      
		InitPartDisc(INIT_DISC_CST_EQ, 0);
		break;
	case INIT_SOLSYST:
		InitSystSol();
		break;
	case INIT_DISC_HOHL:
		InitPartDiscFrom0();
		break;
#if (NB_DIM == 3)
	case INIT_3D_CUBE:
		InitPartCube();
		break;
	case INIT_3D_SOLSYST:
		Init3DSystSol();
		break;
	case INIT_3D_SOLSYST_OBL:
		Init3DSystSolOblique();
		break;
#endif
	default:
		Error(709);
		break;
	}
}

void InitInit()                       // Init of the the galaxy initial shape.
{
  InitPart(INIT_TYPE);      // Init of the particles's location.
}
