/*   INITIALISATION OF THE GROUP OF PARTICLES (initialisation of the "galaxy").
     "Normal" value : INIT_TYPE = INIT_SOLSYST
*/
/**************************************** INITIALISED GALAXY STRUCTURE ***************************************/
/************************************************** TIME ***************************************************/
#if MODE_DEBUG == DEBUG_SOLSYST
  #define REVOL_TIME      YEAR                              // Real galactic revolution time.
  #define DT              (10*DAY)                          // Ca marche avec 10 particules et 256 cellules.
#elif MODE_DEBUG == DEBUG_NORM
  #define REVOL_TIME      REVOL_TIME_MILKYWAY               // Real galactic revolution time.
  #define NBSTEP_REVOL    200                               // 200 for HOHL.
  #define DT              (REVOL_TIME/NBSTEP_REVOL)         // Required slice (step) time for those values above.
#elif MODE_DEBUG == RELEASE
  #define CALC_TIME       (7*MIN)                           // Compute time for one galaxy revolution.
  #define REVOL_TIME      (2*PI/OMEGA0)                     // Galactic revolution time: roughly 10 Million year.
  #define SIMUL_TIME      REVOL_TIME                        // Total real time for this simulation.
  #define STEP_CALC_TIME  0.001                             // In seconds. 
  #define NBSTEP          (CALC_TIME/STEP_CALC_TIME)        // Nb of step for one galaxy revolution.
  #define PRECIS_RATIO    0.0003                            // Used for the calculation of Dt in the code.
  #define NBSTEP_REVOL    200                               // 200 for HOHL.
  #define DT0             (SIMUL_TIME/NBSTEP_REVOL)         // Required slice (step) time for those values above.
  #define DT              DT0                               // Scale adjusting. Already done.
#else
#endif

/* ---------------------------------------- Particle's group ray ------------------------------------------ */
#if MODE_DEBUG == DEBUG_SOLSYST
  #define R_MAX			        EARTH_SUN_DIST
#else
  #if INIT_TYPE == INIT_DISC_HOHL
    #define R_MAX			      R_DISK_HOHL
  #else                                           // Old init types
    #define R_MAX			      R_MILKYWAY
  #endif
#endif

/* ---------------------------------------------- Init types ----------------------------------------------- */
#define INIT_LINE            0						// Line configuration.
#define INIT_SQUARE          1						// Square configuration.
#define INIT_DISC_CST        2						// Disk configuration with constant matter density.
#define INIT_DISC_EXP        3						// Disk configuration with exponential matter density law.
#define INIT_DISC_CST_EQ     4						// Disk conf constant matter density and in equilibrium.
#define INIT_SOLSYST         5                       // Solar system.
#define INIT_DISC_HOHL       6                       // HOHL (1971) type init.
#define INIT_3D_CUBE         7                       // Cube constant matter density.
#define INIT_3D_SOLSYST      8                    // Solar system in a chosen plan in a cube.
#define INIT_3D_SOLSYST_OBL  9                    // Solar system in a chosen plan in a cube.

#if MODE_DEBUG == DEBUG_SOLSYST                     // If test of solar system.
  #if NB_DIM == 3                                   // NB_DIM == 3
    #define INIT_TYPE        INIT_3D_SOLSYST_OBL 	  // Init sol syst here is forced.
  #else                                             // NB_DIM == 2
    #define INIT_TYPE        INIT_SOLSYST	   	      // Init sol syst here is forced.
    #undef OLDVERSION 
  #endif
#else                                               // NON DEBUG_SOLSYST                                      
  #if NB_DIM == 3
    #define INIT_TYPE        INIT_3D_CUBE     
  #else
    #define INIT_TYPE        INIT_DISC_HOHL	   	    // Choose one in the above list.
  #endif
#endif

/* -------------------------------------- Solar system configuration --------------------------------------- */
#define NBPART_SYSTSOL    10          // FOR TESTS   
/* ----------------------------------------- Line configuration ------------------------------------------ */
#define NBPART_LINE		    3
/* ----------------------------------------- Square configuration ------------------------------------------ */
#define NBPART2			      10  
#define NBPART_SQUARE	    (NBPART2 * NBPART2)
/* ----------------------------------------- Cube configuration ------------------------------------------ */
#define DT_CUBE           (DAY)
#define NBPART3			      60  
#define NBPART_CUBE	      (NBPART3 * NBPART3 * NBPART3)
#define PART_DIST_CUBE    EARTH_SUN_DIST
#define PART_MASS_CUBE    MASS_SUN
#define LG_GRID_CUBE      (NBPART3 * PART_DIST_CUBE)
#define SIGMA_POS_CUBE    (PART_DIST_CUBE / 16)
#define SIGMA_V0_CUBE     0
/* ------------------------------------------ Disk configuration ------------------------------------------- */
#define MASS_MILKYWAY_CORR   (1.0*MASS_MILKYWAY)         // Not used for HOHL like init.
#define IR_MAX               218 // HOHL : 10:253; 20:948, 30:2078, 220:102895 part; 218:101429 particles.
                                 // 216 and 217:99474. 2200:10159669. 1000:2 104 356. 500:527576.
#define DR_MIN               (0.75 * PART_DIST)         // dr is minimised for avoid too low dt: 0.039. With this 0.24.                 
/* ----------------------------------- Disk exponential configuration ----------------------------------- */

/* ----------------------------------- DMass and length scales ------------------------------------------ */
/* Caution : these are square roots of scale factors only (in order to avoid sqrt computation)            */
   // Equal to 1 with Mass = MassHOHL. In fact this is sqrt(MASS_SCALE). Equal to sqrt(6.9047)= 2.627691 
#define MASS_SCALE_MILKYWAY   (2.6267851)                   // Measured Milky-Xay mass.  
#define LENGTH_SCALE_MILKYWAY (0.4183*0.577)   // Fitted for the Milky-Way. 0.577 = 1/sqrt(3).
/* ----------------------------------- Fitting the scales ------------------------------------------ */
/* ----------------------------------- Final article's values : 1rst version (ApJ) ----------------- */
//#define MASS_SCALE           (MASS_SCALE_MILKYWAY * 8)       // dissolve.
//#define LENGTH_SCALE         (LENGTH_SCALE_MILKYWAY * 8)     // dissolve. 
#define MASS_SCALE           (MASS_SCALE_MILKYWAY * 1.732)     // Value of the article at 2015/9/8.  
#define LENGTH_SCALE         (LENGTH_SCALE_MILKYWAY * 1.732)   // Value of the article at 2015/9/8.  

/* ---------------------------------------- HOHL disk configuration ------------------------------------- */
#define R_DISK_HOHL_ARTICLE  (8 * KILPARSEC)                          // Refer "III AXISYMMETRIC EVOLUTION".
#define R_DISK_HOHL          (LENGTH_SCALE * LENGTH_SCALE * R_DISK_HOHL_ARTICLE)
#define DIAM_DISK_HOHL       (2*R_DISK_HOHL)
#define TOTAL_MASS_HOHL_0    (100000 * 0.84E6 * MASS_SUN)    // Report legend of Fig 9 of 1971 HOHL article.
                                                             // = MassMilkyWay / 6.90.
#define TOTAL_MASS_HOHL      (MASS_SCALE * MASS_SCALE * TOTAL_MASS_HOHL_0)        // Mass corrective factor.
                                                            // = 6*6 * MassMilkyWay/6.90 = 5.21 MassMW.
#define MU0_HOHL             (3 * TOTAL_MASS_HOHL / (2 * PI * R_MAX * R_MAX))      // Calcul simple integral.
#define OMEGACORR_FORDISP_HOHL  0.809     // Corr multiplictive term for compensat velocity dispersion. HOHL.
#define OMEGA0_HOHL          (OMEGACORR_FORDISP_HOHL * PI * sqrt(G_CONST * MU0_HOHL / (2 * R_MAX)))  // HOHL.
#define MU0                  MU0_HOHL               // Scaling: no need. Already done.              
#define OMEGA0               OMEGA0_HOHL            // Scaling: no need. Already done.  
#define SPEED_GAUSS_SIGMA    0.341                  // Sigma of Gauss disporsion of initial velocities. HOHL.                 
#define TOTAL_MASS           TOTAL_MASS_HOHL 
#define WIDTH_GALAXY_HOHL    (WIDTH_MILKYWAY * R_DISK_HOHL/R_MILKYWAY)     // Supposed proportional.

/************************************ OTHER PARTICLES S CHARACTERISTICS ************************************/
#define PART_MASS		         MASS_SUN                      // Default value only. Not used for HOHL init.	
#define CORR			           0.999                         // For avoiding easily a null denominator.
#define V0				V_EARTH
#if (NB_DIM == 3)
  #define V0_3D			V_EARTH     // TO BE FITTED ?
#endif
#if INIT_TYPE == INIT_DISC_CST_EQ   ||  INIT_TYPE == INIT_DISC_HOHL
  #define PART_DIST			(R_MAX / IR_MAX)
#else
  #define PART_DIST			EARTH_SUN_DIST             // Earth to Sun distance in m. Not enough for a galaxy.
#endif

extern double omega0;   // Here for debug.

/***************************************** MACROS and FUNCTIONS *****************************************/
extern void InitInit();                       // Init of the the galaxy initial shape.
extern void RazSolSyst();
extern double GetSpeedInit(int PartNum);
extern void ReInitPartAccel(long int StepNum);
extern void Cooling();
extern double NormalGauss();