/*   Particles and dynamic
     "Normal" 2D simulation constants:
         MODE_CALCUL_POTENTIAL = MODE_GRID3
         NBCELLX_GRID = 256                   like in HOHl article. Greater values are perf heavy.    
*/
/******************************************* PARTICLES and DYNAMIC *******************************************/
/* ------------------------------------------- Array of particles ------------------------------------------ */
#define NBMAX_PART        12500000   // max 22000000

/******************************************* PARTICLE S STRUCTURE ****************************************/
typedef struct{
	double pos[3];
	double posold[3];
	double posnew[3];
	double acc[3];
	int	   disc;                   // Discarded (for plotting) particle if 1, else 0. But allways calculated.
	double nbpart_in_sphere;       // For STET only. Nb of parts in the sphere of ray rmax around this partic.
	double PosNorm;   
}particle_t;

/********************************************* GRID VARIABLES ********************************************/
/* -------------------------------------------- Grid size ---------------------------------------------- */
/* 256 for HOHL. 32 in 3D is a good compromise. 7*256 executes OK, but not 8*256.                        */
#define NBCELLX_GRID              128   // 128 yields rapid results. 256 = article's value.
#define NBCELLY_GRID              128  
#if (NB_DIM == 3)
  #define NBCELLZ_GRID            64  //64
#else
  #define NBCELLZ_GRID            1   // Enabling to execute 2D with a 3D code.
#endif
                                      //  A la main:
                                      //  64 : Part Num 100000, acc0 = -2.8467920653523880e-012
                                      // 256 :                         -3,92 E-12.
                                      // 512 :                         -4.0096217525494803e-012
#define NBCELL_GRID               (NBCELLX_GRID * NBCELLY_GRID)

/* ----------------------- Location of the grid from respect to the galaxy ---------------------------- */
#if  MODE_DEBUG==RELEASE  ||  MODE_DEBUG==DEBUG_NORM    // TEMPORARY
  #if (NB_DIM == 3)                                     // NB_DIM == 3
    #if (INIT_TYPE == INIT_3D_CUBE)     
      #define LG_GRID               LG_GRID_CUBE
    #else
      #define LG_GRID               (12 * DIAM_MILKYWAY)
    #endif
  #else                                                 // NB_DIM == 2
    #if GRAVIT_MODEL==NEWTON
      #define LG_GRID               (2*24 * DIAM_DISK_HOHL)
    #elif GRAVIT_MODEL==STET
      #define LG_GRID               (2*24 * DIAM_DISK_HOHL)
    #endif
  #endif
#elif  MODE_DEBUG==DEBUG_SOLSYST
  #define LG_GRID                   (1.5 * 2 * R_MAX)   // 0.5*2*R_MAX for testing posit out
#else
#endif

#define LGX_CELL                (LG_GRID / ((double) NBCELLX_GRID))
#define LGY_CELL                (LG_GRID / ((double) NBCELLY_GRID))
#define LGZ_CELL                (LG_GRID / ((double) NBCELLZ_GRID))
#define GRID_ORIGIN0            (- ((double) LG_GRID)/2.) 
#define GRID_ORIGIN1            (- ((double) LG_GRID)/2.) 
#define GRID_ORIGIN2            (- ((double) LG_GRID)/2.) 

/* ----------------------------------- Recentering of the galaxy --------------------------- */
#define MATDENS_MEAN_RAY        ((int) ((LG_GRID/20)/KILPARSEC))   //Used for calcul gal center
                                                // Old val 20. Now 6.72 kpc with article simu.
#define NUMSTEP_RECENTER        500   // Recentering the grid only from times to times.

/* ------------------------ Visible galactic center, for manual recentering  --------------- */
/* ---- Final values for the article: */
#define OFFSET_ORIG0            ((0) * KILPARSEC)      // Used from step 146000 until 150000. 
#define OFFSET_ORIG1            ((0) * KILPARSEC)     // Best profile at 150000.
#define OFFSET_ORIG2            0                        // No 3D for today.            

/* ----------------------------------- Return code of function GetCell() ------------------- */
#define CELL_IN                 0
#define CELL_OUTX               1
#define CELL_OUTY               2
#define CELL_OUTZ               3

/* ------------------------ GRID CALCULATION OPTIMISATION VARIABLES ------------------------ */
/* Caution: there must be the following inclusion:                                           */
/*     SquareRay included into OptimDisk included into GridDisk.                             */
/* For solar system testing: Small SquareRay, an OptimDisk trivial infinitely big.           */ 
/* For example :                                                                             */ 
/*     NBCELLX_GRID = 64                                                                     */
/*     GRIDOPTIMDISK_RAY = (NBCELLX_GRID*2)                                                  */ 
/*     GRIDOPTIMSQUARE_RAY = 4                                                               */ 
#define NBCELL_OPTIM_INCR       3
#define GRIDDISK_RAY            (NBCELLX_GRID/2)      
#define GRIDOPTIMDISK_RAY       (NBCELLX_GRID*2)

/* ---- CAUTION : this value must not be greater than NBCELLX_GRID ------------------------- */
  #define GRIDOPTIMSQUARE_RAY     4      // Still optimized but quite precise results. 3k + 1 values only.

#if (NB_DIM == 3)
  #define NBSMALLCELL_IN_LARGECELL  (NBCELL_OPTIM_INCR * NBCELL_OPTIM_INCR * NBCELL_OPTIM_INCR)
#elif (NB_DIM == 2)
  #define NBSMALLCELL_IN_LARGECELL  (NBCELL_OPTIM_INCR * NBCELL_OPTIM_INCR)
#endif
/* ----- For loop on big cells ----- */
#define MAXIND_BIGCELLX   (((NBCELLX_GRID/2)/NBCELL_OPTIM_INCR) * NBCELL_OPTIM_INCR)
#define MAXIND_BIGCELLY   (((NBCELLY_GRID/2)/NBCELL_OPTIM_INCR) * NBCELL_OPTIM_INCR)
#define MAXIND_BIGCELLZ   (((NBCELLZ_GRID/2)/NBCELL_OPTIM_INCR) * NBCELL_OPTIM_INCR)

/* -------------------------------------------- FFT VARIABLES ------------------------------------- */
/* Those are FFT variables which are used, by the way, even when no FFT calculation is done. That's */
/* for simplification of the code.                                                                  */
#define NBCELLX_FFT            (2*NBCELLX_GRID)                        // Of course for convolution.
#define NBCELLY_FFT            (2*NBCELLY_GRID)                        // Of course for convolution.
#if (NB_DIM == 3)
  #define NBCELLZ_FFT          (2*NBCELLZ_GRID)                        // Of course for convolution.
#else
  #define NBCELLZ_FFT          1
#endif
#define NBCELL_FFT             (NBCELLX_FFT * NBCELLY_FFT * NBCELLZ_FFT)

/******************************************* GLOBAL VARIABLES *******************************************/
/* ----------------------------------------- Engine variables ----------------------------------------- */
extern particle_t Part[NBMAX_PART];
extern double Gravit_dt, Gravit_dt_old;
extern unsigned int NbPart;                             // Now calculated dynamically because of complicated init.
extern double PartMass;
extern double total_time;
extern unsigned int CellNbPart[NBCELLX_FFT][NBCELLY_FFT][NBCELLZ_FFT];
extern double GalCenter[3];
extern double CellMass[NBCELL_FFT]; 
extern unsigned int NbPartOut;

/***************************************** MACROS and FUNCTIONS *****************************************/
#if (NB_DIM == 3)
  #define  VectorCopy(a, b) b[0] = a[0]; b[1] = a[1]; b[2] = a[2]; 
  #define  VectorZero(a) a[0] = a[1] = a[2] = 0;
  #define  VectorAdd(a, b, c) c[0] = a[0] + b[0]; c[1] = a[1] + b[1]; c[2] = a[2] + b[2];
  #define  VectorSub(a, b, c) c[0] = a[0] - b[0]; c[1] = a[1] - b[1]; c[2] = a[2] - b[2];
  #define  VectorDiv(a, b, c) c[0] = a[0]/b; c[1] = a[1]/b; c[2] = a[2]/b;
  #define  VectorMul(a, b, c) c[0] = a[0]*b; c[1] = a[1]*b; c[2] = a[2]*b;
  #define  VectorNorm(a, p_b) *p_b = sqrt(a[0]*a[0] + a[1]*a[1] + a[2]*a[2]);
#else
  #define  VectorCopy(a, b) b[0] = a[0]; b[1] = a[1];
  #define  VectorZero(a) a[0] = a[1] = 0;
  #define  VectorAdd(a, b, c) c[0] = a[0] + b[0]; c[1] = a[1] + b[1];
  #define  VectorSub(a, b, c) c[0] = a[0] - b[0]; c[1] = a[1] - b[1];
  #define  VectorDiv(a, b, c) c[0] = a[0]/b; c[1] = a[1]/b;
  #define  VectorMul(a, b, c) c[0] = a[0]*b; c[1] = a[1]*b;
  #define  VectorNorm(a, p_b) *p_b = sqrt(a[0]*a[0] + a[1]*a[1]);
#endif
extern void InitParticles();
extern void ProcessAcc();
extern void Recalc_dt();
extern void ProcessPosit();
extern void MemorizeDtInOldDt();
extern void DensityCalculate();
extern void GravitPotentialCalculate();
extern void InitStet();
extern void GetAccelFromGalaxyStet(int PartInd, double accel[], unsigned int NumCell[]);
extern void RecenterGrid();
extern unsigned char GetCell(double pos[], unsigned int NumCell[]);
extern void PrintParticlesInfo(FILE *pFile);
