/*   HANDLING STATISTICS ON PARTICLES (stars in a "galaxy").
*/

#include <stdlib.h> 
#include <string.h> 
#include <stdio.h>
#include <math.h>
#include "common.h"
#include "init.h"
#include "particles.h"
#include "hmi.h"
#include "stet.h"
#include "stat.h"

/* --------------------------------- Engine variables ------------------------------- */
particle_stat_t PartStat[NBMAX_PART];
SigmaSpeed_t SigmaRadialSpeed[NB_SLICE];
SigmaSpeed_t SigmaTangentSpeed[NB_SLICE];
double StetNbPart[NB_SLICE];
double SurfMattDens[NB_SLICE];
double GetStetNbPart( int i );

void StatSpeedCalc(SigmaSpeed_t SigmaSpeed[])    // C est la dedans que visual debugger indique le plante
{
  unsigned int i, Ind;

  /* ---------------- Calculating azimuthal mean of radial radial and tangent speed ------- */
  memset(SigmaSpeed, 0, NB_SLICE * sizeof(SigmaSpeed_t));
  memset(StetNbPart, 0, sizeof(StetNbPart));
  for(i = 0; i < NbPart; i++)
  {
    if(MODE_DEBUG != DEBUG_SOLSYST)
    {
      Ind = (unsigned int) (PartStat[i].PosNorm / LG_SLICE);
      if(Ind < NB_SLICE)
      {
        SigmaSpeed[Ind].NbPart++;
        StetNbPart[Ind] += GetStetNbPart(i);    // (oups there was a bugg here)
        if(SigmaSpeed == SigmaRadialSpeed)
        {
          SigmaSpeed[Ind].Mean += PartStat[i].SpeedRadial;
        }
        else
        {
          SigmaSpeed[Ind].Mean += PartStat[i].SpeedTangent;
        }
      }
      else ;
    }
    else ;
  }
  if(MODE_DEBUG != DEBUG_SOLSYST)
  {
    for(Ind = 0; Ind < NB_SLICE ; Ind++)
    {
      if(SigmaSpeed[Ind].NbPart != 0)
      {
        SigmaSpeed[Ind].Mean /= SigmaSpeed[Ind].NbPart;      // Mean value.   se plante en dehors de la
        StetNbPart[Ind] /= SigmaSpeed[Ind].NbPart;
      }
      else
      {
        SigmaSpeed[Ind].Mean = 0;                            // No particles....
        StetNbPart[Ind] = 0;
      }
    }
  }
  else ;  

  /* ---------------- Calculating sigma of azimuthal mean value of radial speed ------- */
  for(i = 0; i < NbPart; i++)
  {
    if(MODE_DEBUG != DEBUG_SOLSYST)
    {
      Ind = (unsigned int) (PartStat[i].PosNorm / LG_SLICE);
      if(Ind < NB_SLICE)
      {
        double RelSpeed; 
        if(SigmaSpeed == SigmaRadialSpeed)
        {
          RelSpeed = PartStat[i].SpeedRadial - SigmaSpeed[Ind].Mean;
        }
        else
        {
          RelSpeed = PartStat[i].SpeedTangent - SigmaSpeed[Ind].Mean;
        }
        SigmaSpeed[Ind].S2 += RelSpeed * RelSpeed;
      }
      else ;    // Just discard if out of grid.
    }
    else ;
  }
  if(MODE_DEBUG != DEBUG_SOLSYST)
  {
    for(Ind = 0; Ind < NB_SLICE ; Ind++)
    {
      if(SigmaSpeed[Ind].NbPart == 0) SigmaSpeed[Ind].S2 = 0;
      else SigmaSpeed[Ind].S2 /= SigmaSpeed[Ind].NbPart;       // Mean value.
      SigmaSpeed[Ind].S2 = sqrt(SigmaSpeed[Ind].S2);           // Variance.
    }
  }
  else ;  
}

void StatCalc()
{
  unsigned int i;
  double Pos[2], RadialVector[2], Speed[2], PosNorm;
      
  /* ---------------- Memorizing radial and tangential speed for further plot --------- */
  for(i = 0; i < NbPart; i++)
  {
    VectorSub(Part[i].pos, GalCenter, Pos);               // Pos = pos(i) - origin.
    VectorNorm(Pos, &PosNorm);                            // PosNorm = ||Pos||
    if(MODE_DEBUG != DEBUG_SOLSYST)
    {
      if(PosNorm == 0) Error(600);                        // Hardly probable.
      else
      {
        VectorDiv(Pos, PosNorm, RadialVector);            // RadialVector = pos/||pos||
      }
    }
    else ; 
    VectorSub(Part[i].pos, Part[i].posold, Speed);        // Speed = pos - posold.
    VectorDiv(Speed, Gravit_dt, Speed);                   // Speed = (pos - posold) / dt.
    PartStat[i].SpeedRadial  = Speed[0] * RadialVector[0] + Speed[1] * RadialVector[1];   // Scalar product v.ur
    PartStat[i].SpeedTangent = - RadialVector[0]*Speed[1] + RadialVector[1]*Speed[0];  // - Vectorial product ur^v.
    PartStat[i].PosNorm = PosNorm;
  }                                                       // End of for loop over the i particles.

  /* ---------------- Calculating speed stats ---------------------------------------- */
  StatSpeedCalc(SigmaRadialSpeed);   // C est la 
  StatSpeedCalc(SigmaTangentSpeed);
}

void PlotSurfacicMatterDensityFile(FILE *pFile)
{
  unsigned int Ind;

  if(MODE_DEBUG != DEBUG_SOLSYST)
  {
    for(Ind = 0; Ind < NB_SLICE ; Ind++)
    {
      double SliceSurface, SurfMattDensity, RadialDist;
      RadialDist = LG_SLICE + LG_SLICE * Ind;
      SliceSurface = 2 * PI * RadialDist * LG_SLICE - PI * LG_SLICE*LG_SLICE;   // Pi (R2 - R1)
      SurfMattDensity = SigmaTangentSpeed[Ind].NbPart * PartMass / SliceSurface;
      SurfMattDens[Ind] = SurfMattDensity;
      RadialDist /= PLOT_DIST_UNIT;
      fprintfPerso(pFile, "%f %f\n", RadialDist, SurfMattDensity );
    }
  }
  else ;    
}

void PlotMeanSurfacicMatterDensityFile(FILE *pFile)
{
  int Ind, NbInd, i;
  double SurfMattDensity, RadialDist;

  if(MODE_DEBUG != DEBUG_SOLSYST)
  {
    for(Ind = 0; Ind < NB_SLICE ; Ind++)
    {
      SurfMattDensity = 0;
      NbInd = 0;
      for(i = -R_MEANSURF; i <= R_MEANSURF ; i++)
      {
        if(Ind+i >= 0)
        {
          SurfMattDensity += SurfMattDens[Ind+i];
          NbInd++;
        }
        else ;
      }
      if(NbInd == 0) Error(601);
      else SurfMattDensity /= NbInd;
      RadialDist = LG_SLICE + LG_SLICE * Ind;
      RadialDist /= PLOT_DIST_UNIT;
      fprintfPerso(pFile, "%f %f\n", RadialDist, SurfMattDensity );
    }
  }
  else ;    
}

void PlotLogSurfacicMatterDensityFile(FILE *pFile)
{
  unsigned int Ind;

  if(MODE_DEBUG != DEBUG_SOLSYST)
  {
    for(Ind = 0; Ind < NB_SLICE ; Ind++)
    {
      double SliceSurface, SurfMattDensity, RadialDist;
      RadialDist = LG_SLICE + LG_SLICE * Ind;
      SliceSurface = 2 * PI * RadialDist * LG_SLICE - PI * LG_SLICE*LG_SLICE;   // Pi (R2 - R1)
      SurfMattDensity = SigmaTangentSpeed[Ind].NbPart * PartMass / SliceSurface;
      SurfMattDens[Ind] = SurfMattDensity;
      RadialDist /= PLOT_DIST_UNIT;
      fprintfPerso(pFile, "%f %f\n", RadialDist, ADDED_LOG_MATD + log(SurfMattDensity) );
    }
  }
  else ;    
}

void PlotLogMeanSurfacicMatterDensityFile(FILE *pFile)
{
  int Ind, NbInd, i;
  double SurfMattDensity, RadialDist;

  if(MODE_DEBUG != DEBUG_SOLSYST)
  {
    for(Ind = 0; Ind < NB_SLICE ; Ind++)
    {
      SurfMattDensity = 0;
      NbInd = 0;
      for(i = -R_MEANSURF; i <= R_MEANSURF ; i++)
      {
        if(Ind+i >= 0)
        {
          SurfMattDensity += SurfMattDens[Ind+i];
          NbInd++;
        }
        else ;
      }
      if(NbInd == 0) Error(602);
      else SurfMattDensity /= NbInd;
      RadialDist = LG_SLICE + LG_SLICE * Ind;
      RadialDist /= PLOT_DIST_UNIT;
      fprintfPerso(pFile, "%f %f\n", RadialDist, ADDED_LOG_MATD + log(SurfMattDensity) );
    }
  }
  else ;    
}

void PlotRadSigmaFile(FILE *pFile)
{
  unsigned int Ind;

  if(MODE_DEBUG != DEBUG_SOLSYST)
  {
    for(Ind = 0; Ind < NB_SLICE ; Ind++)
    {
      double Speed;
      double RadialDist = LG_SLICE + LG_SLICE * Ind;
      RadialDist /= PLOT_DIST_UNIT;
      Speed = SigmaRadialSpeed[Ind].S2;
      Speed /= PLOT_SPEED_UNIT;
      fprintfPerso(pFile, "%f %f\n", RadialDist, Speed );
    }
  }
  else ;    
}

void PlotTanSigmaFile(FILE *pFile)
{
  unsigned int Ind;

  if(MODE_DEBUG != DEBUG_SOLSYST)
  {
    for(Ind = 0; Ind < NB_SLICE ; Ind++)
    {
      double Speed;
      double RadialDist = LG_SLICE + LG_SLICE * Ind;
      RadialDist /= PLOT_DIST_UNIT;
      Speed = SigmaTangentSpeed[Ind].S2;
      Speed /= PLOT_SPEED_UNIT;
      fprintfPerso(pFile, "%f %f\n", RadialDist, Speed );
    }
  }
  else ;    
}

void PlotTanMeanFile(FILE *pFile)
{
  unsigned int Ind;

  if(MODE_DEBUG != DEBUG_SOLSYST)
  {
    for(Ind = 0; Ind < NB_SLICE ; Ind++)
    {
      double Speed;
      double RadialDist = LG_SLICE + LG_SLICE * Ind;
      RadialDist /= PLOT_DIST_UNIT;
      Speed = SigmaTangentSpeed[Ind].Mean;
      Speed /= PLOT_SPEED_UNIT;
      fprintfPerso(pFile, "%f %f\n", RadialDist, Speed );
    }
  }
  else ;    
}

void PlotMeanTanMeanFile(FILE *pFile)
{
  int Ind, NbInd, i;
  double Speed, RadialDist;

  if(MODE_DEBUG != DEBUG_SOLSYST)
  {
    for(Ind = 0; Ind < NB_SLICE ; Ind++)
    {
      Speed = 0;
      NbInd = 0;
      for(i = -R_MEAN; i <= R_MEAN ; i++)
      {
        if(Ind+i >= 0)
        {
          Speed += SigmaTangentSpeed[Ind+i].Mean;
          NbInd++;
        }
        else ;
      }
      if(NbInd == 0) Error(603);
      else Speed /= NbInd;
      RadialDist = LG_SLICE + LG_SLICE * Ind;
      RadialDist /= PLOT_DIST_UNIT;
      Speed /= PLOT_SPEED_UNIT;
      fprintfPerso(pFile, "%f %f\n", RadialDist, Speed );
    }
  }
  else ;    
}

void PlotStetNbPartFile(FILE *pFile)
{
  int Ind;
  double RadialDist;

  if(MODE_DEBUG != DEBUG_SOLSYST)
  {
    for(Ind = 0; Ind < NB_SLICE ; Ind++)
    {
      RadialDist = LG_SLICE + LG_SLICE * Ind;
      RadialDist /= PLOT_DIST_UNIT;
      fprintfPerso(pFile, "%f %f\n", RadialDist, StetNbPart[Ind] );
    }
  }
  else ;    
}

#if (GRAVIT_MODEL==STET)
  double GetStetNbPart( int i )
  {
    unsigned char Cr;
    unsigned int NumCell[3];
    Cr = GetCell(Part[i].pos, NumCell);
    if(Cr != CELL_IN)
    {
      return( 0 );
    }
    else
    {
      return( (double) NbPartInStetDebug[NumCell[0]][NumCell[1]][NumCell[2]] );
    }
  }
#else
  double GetStetNbPart(int i) { return(0); }
#endif

void PrintStatInfo(FILE *pFile)
{
  #if GRAVIT_MODEL==STET
    // fprintfPerso(pFile, "\n");
  #else
  #endif
}

/* -------------- Testing GetCell code in Unit Test ---------------- */
void TestStat()       // Teste OK le 29 sept 2014.
{
  double Pos[3], Incr;
  unsigned int NumCell[3];
  unsigned int NbIn=0, NbOut=0, NbP=0, CR;
    
  GalCenter[0] = 120 * LG_GRID;
  GalCenter[1] = GalCenter[2] = -7879.7 * LG_GRID;

  Incr = LGX_CELL/1;
  for( Pos[0] = GalCenter[0]-LG_GRID/2 ; Pos[0] <= GalCenter[0]+LG_GRID/2; Pos[0]+=Incr)
  {
    for( Pos[1] = GalCenter[1]-LG_GRID/2 ; Pos[1] <= GalCenter[1]+LG_GRID/2; Pos[1]+=Incr)
    {
      NbP++;
      CR = GetCell(Pos, NumCell);
      if(CR == CELL_IN) NbIn++;
      else NbOut++;
    }
  }
  GalCenter[0] = GalCenter[1] = GalCenter[2];    
}
