/*   Calculations of simulated galaxy characteristics. 
*/
/*************************************** PARTICLE'S STAT STRUCTURE **************************************/
typedef struct{
  double SpeedRadial;
  double SpeedTangent;
	double PosNorm;       // Distance of the particle from the centroid of the galaxy.
}particle_stat_t;

/*************************************** CALCULATION OF SIGMA function **********************************/
/* ------ number and positions of the slices of the ray ------- */
#define NB_SLICE_PERGRID      1000                            // Nb of slices in a grid ray.
#define SLICE_NBGRID          50                              // Lg of last slice given in nb of grid ray. 
#define NB_SLICE              (SLICE_NBGRID * NB_SLICE_PERGRID)        // Total nb of slices.
#define LG_SLICE              ((SLICE_NBGRID * LG_GRID / NB_SLICE) /2) // "/2" because only a ray is plotted. 
/* -------------------------------------- radial slice structure -------------------------------------- */
typedef struct          // Speed is radial speed OR tangent speed.
{
  double Mean;          // Mean = Mean value of speed in this slice.
  double S2;            // sigma = Square_root_of( (SIGMA of (Speed - Mean)) / NbPart )
  unsigned int NbPart;  // Nb particles in this slice.
}SigmaSpeed_t;

/******************************************** Mean values ***********************************************/
#define R_MEANSURF            100   // => mean over 200 * LG_SLICE = 
                                    // 200 * ((50 * (2 * 24 * (2 * 1.4 kpc)) / 50000) /2)  = 13.4 kpc.
#define R_MEAN                20                                        // Mean curve.

/******************************************** LOG matter density ****************************************/
#define ADDED_LOG_MATD        8    // (log of) Multiplicative constant for LogMatdD

/******************************************* GLOBAL VARIABLES *******************************************/
/* ----------------------------------------- Engine variables ----------------------------------------- */
extern particle_stat_t PartStat[NBMAX_PART];
extern SigmaSpeed_t SigmaRadialSpeed[NB_SLICE];
extern SigmaSpeed_t SigmaTangentSpeed[NB_SLICE];

/***************************************** MACROS and FUNCTIONS *****************************************/
extern void StatCalc();
extern void PlotRadSigmaFile(FILE *pFile);
extern void PlotTanSigmaFile(FILE *pFile);
extern void PlotTanMeanFile(FILE *pFile);
extern void PlotSurfacicMatterDensityFile(FILE *pFile);
extern void PlotMeanSurfacicMatterDensityFile(FILE *pFile);
extern void PlotLogMeanSurfacicMatterDensityFile(FILE *pFile);
extern void PlotLogSurfacicMatterDensityFile(FILE *pFile);
extern void PlotMeanTanMeanFile(FILE *pFile);
extern void PlotStetNbPartFile(FILE *pFile);
extern void PrintStatInfo(FILE *pFile);
