/*   STET's modification of NEWTON's law.
     Only STET_SIMPLE mode is used. STET_MODE = STET_SIMPLE 
*/

#include <stdlib.h> 
#include <string.h> 
#include <stdio.h>
#include <math.h>
#include "common.h"
#include "hmi.h"
#include "init.h"
#include "particles.h"
#include "period.h"
#include "stet.h"


/******************** GLOBAL VARIABLES USED IN THIS FILE ********************/
#if GRAVIT_MODEL==STET
double StetCorr;
double StetCorrMin = 1.E300;
double StetCorrMax = 0;
double StetNbPartInit;              // Nb of particles in a STET disk at init time.
double IntAddedNbPart;
double NbPartIGM;
unsigned char StetDisk[NBCELLX_STETDISK][NBCELLY_STETDISK][NBCELLZ_STETDISK];
double StetR[NBCELLX_FFT][NBCELLY_FFT][NBCELLZ_FFT];
double StetCorrMemo[NBCELLX_STETDISK][NBCELLY_STETDISK][NBCELLZ_STETDISK];
unsigned int NbPartInStetDebug[NBCELLX_STETDISK][NBCELLY_STETDISK][NBCELLZ_STETDISK];
unsigned long int StepNumFinal, StepNumSatur;
  
  #if STET_MODE == STET_COMPLEX
    double StetAddedNbPart;
    double StetAddedDensity;
    double StetSolSystContrib;
    double StetAddedContrib;
    double StetIGMContrib;
    unsigned int StetNbPart[NBCELLX_GRID][NBCELLY_GRID][NBCELLZ_GRID];  // DEBUG ONLY
    double StetCorrPart[NBMAX_PART];  // For PositOut: particle out of grid.
  #endif


/************************* Complex or Simplified STET ? **********************/


/********************************* SIMPLE STET *******************************/
#if STET_MODE == STET_SIMPLE 
  #define InsideStetDisk(ix, iy, iz, ajx, ajy, ajz)   (StetDisk[ajx][ajy][ajz])

/* --- Progressively passing from Newton to STET: engine function ---------- */
double GetAddedNbPart(double IntAddedNbPartIn, double MaxAddedNbPart)
{
  double Num, Denum, DeltaStetCorr, Cstet0;
  double IntAddedNbP, delta;

  if(StetNbPartInit == 0) Error(500);
  else
  {
    Cstet0 = (StetNbPartInit + MaxAddedNbPart) / MaxAddedNbPart;
    DeltaStetCorr = pow((((double) MAX_STET_CORR)/Cstet0), 
                        ((double) 1/(MAX_STEP_NUM))) - 1;
    Num = (StetNbPartInit + IntAddedNbPartIn) * IntAddedNbPartIn * DeltaStetCorr;  
    Denum = StetNbPartInit + (StetNbPartInit + IntAddedNbPartIn) * DeltaStetCorr;
                                   // Supposing Contrib = 0 (out of galaxy). 
    delta = Num/Denum;                 // Constant delta for StetCorr.  
    if(delta > IntAddedNbPartIn) Error(501);     // Bugg in the calculations.
    else if(delta > 0) IntAddedNbP = IntAddedNbPartIn - delta; // Decrease it
    else  Error(502);                        // Impossible with double floats. 
  }
  if(IntAddedNbP > ADDEDNBP_1) Error(503);          // This is a bugg.
  return(IntAddedNbP);                              // Decreased if possible.
}

/* --- Progressively passing from Newton's mode into STET SIMPLE mode ----- */
void CalcStetAddedNbPart()
{
  double RatioNbP;

  if(StepNum < NUMSTEP_1)
  {
    /* NEWTON LAW at first. */
  }
  else
  {                            // then STET.
    /********************** Calculating IntAddedNbPart ************************/
    /* --- Progressively passing from Newton's mode into STET SIMPLE mode - */
   IntAddedNbPart = GetAddedNbPart(IntAddedNbPart, ADDEDNBP_1);        
    /* --- Detecting end of progression ---------------------------------- */
                  // C0 = alpha Cadded. Mandatory for explaining Zwicky issues.
    RatioNbP = StetNbPartInit / IntAddedNbPart;
    if(StepNumFinal != 0)
    {
      IntAddedNbPart = STET_ADDED_CONTRIB_RATIO_OVERSOLSYST * StetNbPartInit;
    }                                    // = 80 000 / 39 = 2051 
    else                                 // StepNumFinal != 0, nominal case.
    {
      if(RatioNbP >= 1/STET_ADDED_CONTRIB_RATIO_OVERSOLSYST)     // 39
      {
        IntAddedNbPart = STET_ADDED_CONTRIB_RATIO_OVERSOLSYST * StetNbPartInit;        
        StepNumFinal = StepNum;          // For printing only.
      }
      else /*NOP*/;
    }
  }
}

double GetStetCorrSimple(unsigned int NbPartInStet)
{
  double StetCorr;
  int OffsetNum; 
  double NbPartM;
  double Num, Denum, UnSurDenum; 

  if(StepNum < NUMSTEP_1)
  {
    StetCorr = 1.;             // NEWTON LAW at first.
  }
  else
  {                            // then STET.
    /*************** Calculating StetCorr of STET SIMPLE mode **************/
    /* ------------------------ Specific case ---------------------------- */
    /* Specific case in the small windows used for deleting ring structures*/
    OffsetNum = StepNum % STEPNUM_STOP_RING;                
    if(STEPNUM_STOP_RING_WIND1 < OffsetNum  &&  OffsetNum < STEPNUM_STOP_RING_WIND2)
    {                   // This is only for trying to delete ring structures.
      StetCorr = (11 * StetCorrMin + StetCorrMax)/12;   // Try to give a short 
    }  // gravitation imppulse by returning back Newtons's law a little while.
    /* ---------------------- General, normal case ----------------------- */
    else
    {           // !!!!!!!!!!!! HERE IS STET MODE CALCULATION !!!!!!!!!!!!!!!
      Num = StetNbPartInit + IntAddedNbPart;   // (alpha+1) Cadded
      NbPartM = max(NbPartInStet, NbPartIGM);          // Here MAX CSTET = 5
   //   NbPartM = NbPartInStet;                        // Here MAX CSTET = 40
      Denum = NbPartM + IntAddedNbPart;    // = (alpha+1)Cadded/(C+Cadded)
      StetCorr = Num / Denum;
    }            // !!!!!!!!!!!! END OF STET MODE CALCULATION !!!!!!!!!!!!!!!
    /* ------------- Handling FP compilations (or is it hardware?) errors -*/
    if(StetCorr < 0.1 || StetCorr > 50)
    {                          // Trying to avoid floatting point big errors.
      UnSurDenum = 1/Denum; 
      StetCorr = Num * UnSurDenum;
      if(StetCorr < 0.01 || StetCorr > 100)
      {
        Error(504);           // Now floating point errors cant be avoided.
      }
    }
  }

  /************************** Memo for printf ******************************/
  StetCorrMin = min(StetCorrMin, StetCorr);
  StetCorrMax = max(StetCorrMax, StetCorr);
  return(StetCorr);
}
/*************************** END OF SIMPLE STET ******************************/


/********************************* COMPLEX STET ******************************/
#elif STET_MODE == STET_COMPLEX                                  // DEPRECATED.

  unsigned char InsideStetDisk(unsigned int ix, unsigned int iy, unsigned int iz, 
                              unsigned int ax, unsigned int ay, unsigned int az)
  {
    double x = ax * LGX_CELL;  
    double y = ay * LGY_CELL; 
    double z = az * LGZ_CELL; 
    return( x*x + y*y + z*z  <=  StetR[ix][iy][iz] * StetR[ix][iy][iz] );
  }

  double GetNGas(double Rho)   // May be to modify with surfacic matter density later on.
  {
    double Ngas, Ngalaxy; 

    if(Rho < RHO_A)      Ngalaxy = 0;
    else if(Rho > RHO_B) Ngalaxy = NGAS_MAX;
    else
    {
      Ngalaxy =  1 - exp(-(Rho-RHO_A)*(Rho-RHO_A)/SIGMA2R0);
      Ngalaxy /= 1 - exp(-DIFF_RHOAB*DIFF_RHOAB/SIGMA2R0); 
      Ngalaxy *= NGAS_MAX * exp(-(Rho-RHO_B)*(Rho-RHO_B)/SIGMA2R0); 
    }
    Ngas = N_IGM + Ngalaxy;    // 10:140694880.   1:20099999999.9   5:657191988.6   2.5:3279204898.  2.:5476560917.
    return( Ngas );            // NO_GAL_AROUND_SUN   (0.67E4/CM3)  
  }                                  

  // BEFORE this call: Calculation of NbPartInStet in Stet sphere of RAY equal
  // to StetR[ix][iy][iz], which has been calculated the step before in this 
  // function (just below). 
  double GetStetCorrComplex(unsigned int NbPartInStet, unsigned int ix, unsigned int iy, unsigned int iz) 
  {
    double StetCorr, NGas, StetRext, Stetr, StetDensity, Contrib;
    double StetSurface0, StetSurface, StetSurfaceIgm, Rho, RhoIgm;

    /******* Calculating StetCorr of STET COMPLEX mode **********/
    /* ---- Gas density calculation ------ */
    if(StetNbPartInit == 0) Error(505);
    else if(StetCorrMemo[ix][iy][iz] == 0) Error(506);
    else
    {
      StetSurface0 = R0_GAL_AROUND_SUN * R0_GAL_AROUND_SUN;
      StetSurface = StetR[ix][iy][iz] * StetR[ix][iy][iz];
      StetDensity = RHO_GAL_AROUND_SUN * (StetSurface0/StetNbPartInit) * NbPartInStet /StetSurface  ;
    }
    NGas = GetNGas(StetDensity);  // NbPartInStetInit -> RHO_GAL_AROUND_SUN(0.708E-20) -> NO_GAL_AROUND_SUN(0.67E4/CM3)

    /* ---- Rext and R calculations ------ */
    StetRext = LN_2 / (SIGMA_EXT * NGas);
    Stetr = min(StetRext, STET_R_MAX);                       // Max STET_R_MAX=E21

    /* ---- Contrib calculation ------ */
    Contrib = (double) NbPartInStet;   // Contrib = rho*R = PartMass*(NbPartInStet/R)*R 
                                       // = PartMass * NbPartInStet proportional to NbPartInStet     
    /* ---- Contrib modification after rho saturation ------ */
    /* Because out of the galaxy its not void, but IGM. */
    Rho = Contrib / StetSurface;
    StetSurfaceIgm = STET_R_MAX * STET_R_MAX;
    RhoIgm = StetIGMContrib / StetSurfaceIgm;
    Rho = max(Rho, RhoIgm);            // Rho saturation by RhoIgm.
    Contrib = Rho * StetSurface;       // Out of the galaxy its not void, but IGM.
    
    /* ---- STET corrective factor calculation ------ */
    StetCorr  =  StetSolSystContrib + StetAddedContrib;      // 
    StetCorr /=  Contrib            + StetAddedContrib;      // 

    /* ---- Memorizing all that ------ */
    StetCorrMemo[ix][iy][iz] = StetCorr;
    StetR[ix][iy][iz] = Stetr;
    StetNbPart[ix][iy][iz] = NbPartInStet;  // DEBUG only.

    /******* Progressively passing from Newton's mode into STET COMPLEX mode *******/
    if(StepNum < NUMSTEP_2) StetCorr = 1;
    else
    {
      double a;
      if(StepNum > NUMSTEP_3) a = 1;
      else a = ((double)(StepNum - NUMSTEP_2))/((double)(NUMSTEP_3 - NUMSTEP_2));
      StetCorr = a*(StetCorr - 1) + 1;
    }   

    return(StetCorr);
  }
#else    // Nothing there. 
#endif 
/*************************** END OF COMPLEX STET *****************************/


/**************** COMMMON ENGINE TO SIMPLE AND COMPLEX MODES *****************/
double GetStetCorr(unsigned int NbPartInStet, unsigned int ix, unsigned int iy, unsigned int iz)
{
  #if STET_MODE == STET_SIMPLE 
    return( GetStetCorrSimple(NbPartInStet) );
  #elif STET_MODE == STET_COMPLEX
    return( GetStetCorrComplex(NbPartInStet, ix, iy, iz) );
  #else 
  #endif 
}

void MemorizeStetCorr(int PartInd, unsigned int NumCell[])      
{
  #if STET_MODE == STET_COMPLEX
    StetCorrPart[PartInd] = StetCorrMemo[NumCell[0]][NumCell[1]][NumCell[2]];
  #else
    // StetCorr has a constant value in STET_SIMPLE mode.
  #endif
}

double GetStetCorrOut(int PartInd, unsigned int NumCell[])
{
  #if STET_MODE == STET_SIMPLE 
    return( GetStetCorrSimple(0) );
  #elif STET_MODE == STET_COMPLEX
    return( StetCorrPart[PartInd] );   // Last memorized value in PositIn state.
  #else 
  #endif 
}
/**************** END OF COMMMON ENGINE TO SIMPLE AND COMPLEX MODES *********/


/***************************** STET INIT ************************************/
void ReInitStet()
{
  StetCorrMax = 0;
  StetCorrMin = 1.E300;
}

void InitStet()
{
	unsigned int i, j, jx, jy, jz;
  double m0_over_m, h_over_h0;
  
  IntAddedNbPart = ADDEDNBP_1;
  StepNumSatur = 0;
  StepNumFinal = 0;
  StetCorrMax = 0;
  StetCorrMin = 1.E300;
  for(i=0 ; i<NBCELLX_FFT ; i++) for(j=0 ; j<NBCELLY_FFT ; j++)
  {
    StetR[i][j][0] = STET_R_MAX;
    StetCorrMemo[i][j][0] = 1;
  }
  
  /* ------- Init of StetNbPartInit, corresponds to local mass density ---- */
  if(MODE_DEBUG == DEBUG_SOLSYST)
  {
    StetNbPartInit = NbPart;
  }
  else
  {
    #if NB_DIM == 3
          // StetNbPartInit = (unsigned int) ((2*STET_R_MAX/LG_GRID) * (2*STET_R_MAX/LG_GRID) * (2*STET_R_MAX/LG_GRID) * NbPart);
      StetNbPartInit = (unsigned int) ((4/3)*PI*((STET_R_MAX*STET_R_MAX*STET_R_MAX) / (LG_GRID*LG_GRID*LG_GRID)) * NbPart);
    #else                                   // NB_DIM == 2
      #if STET_MODE == STET_SIMPLE 
        StetNbPartInit = (unsigned int) (RATIO_STET_NBPART_INIT * ((double) NbPart));  
        m0_over_m = (MASS_SCALE_MILKYWAY / MASS_SCALE) * (MASS_SCALE_MILKYWAY / MASS_SCALE);
        h_over_h0 = (LENGTH_SCALE / LENGTH_SCALE_MILKYWAY) * (LENGTH_SCALE / LENGTH_SCALE_MILKYWAY);
          // h_over_h0 = 1;    // Which line of code is the most realistic ??
        StetNbPartInit = (unsigned int) ((m0_over_m * h_over_h0) * ((double) StetNbPartInit));
        NbPartIGM = (unsigned int) (STET_IGM_CONTRIB_RATIO_OVERSOLSYST * StetNbPartInit);
      #elif STET_MODE == STET_COMPLEX 
        StetNbPartInit = (unsigned int) (NbPart * RATIO_NBPART_INIT_OVER_NBPART);    // Because : if NbPart=100000, StatNbPart max = 80000. Takes the middle.
        StetSolSystContrib = (double) StetNbPartInit;        
        StetAddedContrib = STET_ADDED_CONTRIB_RATIO_OVERSOLSYST * StetSolSystContrib;
        StetIGMContrib = STET_IGM_CONTRIB_RATIO_OVERSOLSYST * StetSolSystContrib;
      #endif              // #if STET_MODE == STET_SIMPLE 
    #endif
  }
  /* ------- Init of StetDisk, in which mass density is calculated --------- */
  /* StetDisk is used for performances only in potential.c file via          */
  /* InsideStetDisk macro.                                                   */
	memset(StetDisk, 0, sizeof(StetDisk));
	for(jy=0 ; jy<NBCELLY_GRID ; jy++)
	{
		for(jx=0 ; jx<NBCELLX_GRID ; jx++)
		{
  		for(jz=0 ; jz<NBCELLZ_GRID ; jz++)
  		{
  			double x, y, z;
  			x = jx * LGX_CELL;
  			y = jy * LGY_CELL;
  			z = jz * LGZ_CELL;
  			if(x*x + y*y + z*z  <=  STET_R_MAX * STET_R_MAX)
  			{
  				StetDisk[jx][jy][jz] = 1;                 // This is inside the disk.
        } else ;                                    // Outside the disk.
      }
		}
	}
}
/***************************** END OF STET INIT *****************************/


/*********************** SPECIFIC STET PLUG TO HMI **************************/
void PrintStetInfo(FILE *pFile)
{
  double Ratio;
  double RMaxRat = STET_R_MAX/LG_GRID;
  fprintfPerso(pFile, " MinMaxC=%f,%f, RMaxR=%f, IntAddedNbPart=%f", 
                  StetCorrMin, StetCorrMax, RMaxRat, IntAddedNbPart);
  Ratio = ((double) StetNbPartInit) / ((double) IntAddedNbPart);
  fprintfPerso(pFile, " NbPInit=%f, Ratio=%f, Final=%d",  
                StetNbPartInit, Ratio, StepNumFinal);                  
}
/*********************** END OF SPECIFIC STET PLUG TO HMI ********************/


/******************************* NEWTON CASE *********************************/
#else                                        // NEWTON case (for linking only)

  void InitStet(){}
  void ReInitStet(){}
  InitStetFromReadConfig(isReadConfig);   
  double GetStetCorr(unsigned int NbPartInStet, unsigned int ix, unsigned int iy, unsigned int iz){return(0);}
  void CalcStetAddedNbPart() {}
  void MemorizeStetCorr(int PartInd, unsigned int NumCell[]) {}      
  void PrintStetInfo(FILE *pFile) {}

#endif                                       // end of #if GRAVIT_MODEL==STET
/************************** END OF NEWTON CASE *******************************/
