/*   STET's modification of NEWTON's law.
     Only STET_SIMPLE mode is used : STET_MODE = STET_SIMPLE. 
*/
/* ----------------------------- STET model ------------------------------- */
#if GRAVIT_MODEL == STET
  #if (NB_DIM == 3)
    #define STET_R_MAX            (LG_GRID/16.)        // DIM 3
  #else               
    #define STET_R_MAX            (15 * KILPARSEC)     // DIM 2
  #endif

  #define STET_DISK_SURF          (PI * STET_R_MAX * STET_R_MAX)      //
  #define FINAL_GAL_RAY           (LG_GRID/2)     // A bit strong for evaluating the equivalent location of sun.
  #define FINAL_GAL_SURF          (PI * FINAL_GAL_RAY * FINAL_GAL_RAY)      //
  #define RATIO_STET_NBPART_INIT  0.75             // Whatched on MW simulated.

  /* ---------------- For calculation of StetAddedContrib ------------------ */
  #define STET_ADDED_CONTRIB_RATIO_OVERSOLSYST  0.0256410256410256410256410    // 1/39=Cadded/C0
  #define STET_IGM_CONTRIB_RATIO_OVERSOLSYST    0.1794871794871794871794871    // 7/39=Cigm/C0

  /* ---------------------------- STET mode -------------------------------- */
  #if STET_MODE == STET_COMPLEX                 // This is DEPRECATED

    /* ------------ For calculation of extinction mechanism ---------------- */
    #define SIGMA_EXT                      1.E-28             // 1/m3  (E-28 m-3 = 1 barn)
    #define RHO_GAL_AROUND_SUN             0.708E-20          // kg/m3
    #define NO_GAL_AROUND_SUN              (0.67E4/CM3)       // in m-3
    #define LN_2                           0.69314718055994   // ...
    #define REXT0_GAL_AROUND_SUN           (LN_2 / (SIGMA_EXT * NO_GAL_AROUND_SUN))    // in m.
    #define R0_GAL_AROUND_SUN              REXT0_GAL_AROUND_SUN
    #define RATIO_N_RHO                    (NO_GAL_AROUND_SUN / RHO_GAL_AROUND_SUN)              
    #define N_MAX                          (50 * NO_GAL_AROUND_SUN)              
   
    /* ------ For calculation of StetNbPartInit --------------------------- */
    #define STET_SPHERE_VOL             ((4/3) * PI * STET_R_MAX * STET_R_MAX * STET_R_MAX)      //
    #define GALACTIC_WIDTH              WIDTH_GALAXY_HOHL             // If mode init = disk HOHL.
    #define STET_GALCYL_VOL             (STET_DISK_SURF * GALACTIC_WIDTH)      //
    #define R_MAX_FINAL_GALAXY          (R_MAX * 15)          // Ray of simulated galaxy in final permanent regime. 
                                                            // Difficult to estimate precisely.
    #define STET_ADDED_NBPART           0.06    // Has been calculated with ISM rho value.     
    #define RATIO_N_RHO                 (NO_GAL_AROUND_SUN / RHO_GAL_AROUND_SUN)    // roughtly E26             
    #define NGAS_MAX                    (3 * NO_GAL_AROUND_SUN)                 // simu 3 simu 4     
    /* ------- For a complicated N = N(rho), using exp function (better) -- */
    #define NGAS_ALPHA                  0.5 
    #define NGAS_GAMMA                  2. 
    #define NGAS_K                      (NGAS_ALPHA * log(NGAS_GAMMA) / RHO_GAL_AROUND_SUN) 
                         // NO_GAL_AROUND_SUN = roughtly GetNGas(RHO_GAL_AROUND_SUN)
    /* ------ For a complicated N = N(rho), using tan function (deprecated) */
    #define RATIO_LIMIT                 0.01
    #define RHO_LIMIT                   (RATIO_LIMIT * RHO_GAL_AROUND_SUN)
    #define RATIO_LOWDELTA              0.0000001  // OK : 1567/1500    
    #define RHO_LOWDELTA                (RATIO_LOWDELTA * RHO_LIMIT)
    /* -------- For calculation of N = N(rho), brutal exponential ---------- */
    #define RHO_A       (0.5 * RHO_GAL_AROUND_SUN)          // 
    #define RHO_B       (2. * RHO_GAL_AROUND_SUN)              //
    #define DIFF_RHOAB  (RHO_B - RHO_A)                        // 
    #define SIGMA2R0    (DIFF_RHOAB * DIFF_RHOAB)
    /* ---------------------------- Miscellaneous -------------------------- */
    #define LGMAX_STETR         (NBCELLX_GRID * NBCELLY_GRID * NBCELLZ_GRID)
    #define STET_ADDED_MASS     (STET_GALCYL_VOL * RHO_IGM)      

    /* ------------- Progressive decrease of Added Nb Part ----------------- */
    #define NUMSTEP_2         10000  // 
    #define NUMSTEP_3         50000  //

    /* ------------- VERY IMPORTANT ---------------------------------------- */
    /* ------------- Corresponding NbPartInSTETSphere FOR RhoSolSyst ------- */
    #define RATIO_NBPART_INIT_OVER_NBPART   0.09  // deprecated
    /* ------------- VERY IMPORTANT ---------------------------------------- */

  #elif STET_MODE == STET_SIMPLE 

    /*************************** NTET progressivity **************************/
    /* ------------- Progressive decrease of Added Nb Part ----------------- */
    #define COEFF_NB_PART   1  //3 times faster with E5 partic than with E7 part
    #define ADDEDNBP_1      (1000000 * COEFF_NB_PART)    // First NbProg value.
    /* ------------- Progressive decrease of Added Nb Part ----------------- */
    #define NUMSTEP_1         10000   // End of 1rst phase / begin of 2nd phase.

    #define MAX_STET_CORR        5  //40       // Max possible val of StetCorr.
                                      // 5 if max in equations, 40 if no max().
    #define MAX_STEP_NUM         140000 // 70000 =>205000; 140000=> 320000 for MW
                                  // 300000=>More than 320000 but no more rings. 

    /********************* Trying to delete ring structures ******************/
    #define STEPNUM_STOP_RING        10000000 // NO stop ring !! //50000  
    #define STEPNUM_STOP_RING_WIND1  100  
    #define STEPNUM_STOP_RING_WIND2  500  // That is roughly 2 revolutions.  

  #else
  #endif                      // #if STET_MODE == STET_COMPLEX

  /* --------- StetDisk[] dimension, for rapid calculation of location in Stet disk ----------- */
  #define NBCELLX_STETDISK            (2 * NBCELLX_GRID)
  #define NBCELLY_STETDISK            (2 * NBCELLY_GRID)
  #if (NB_DIM == 3)
    #define NBCELLZ_STETDISK          (2 * NBCELLZ_GRID)
  #else
    #define NBCELLZ_STETDISK          1
  #endif

  /***************************************** GLOBAL VARIABLES *****************************************/
  extern double StetNbPartInit;              // Nb of particles in a STET disk at init time.
  extern double IntAddedNbPart;
  extern unsigned int NbPartInStetDebug[NBCELLX_STETDISK][NBCELLY_STETDISK][NBCELLZ_STETDISK];
  extern unsigned char StetDisk[NBCELLX_STETDISK][NBCELLY_STETDISK][NBCELLZ_STETDISK];
  extern double StetCorrMemo[NBCELLX_STETDISK][NBCELLY_STETDISK][NBCELLZ_STETDISK];
  extern double StetCorrMin;
  extern double StetCorrMax;

#endif                   // #if GRAVIT_MODEL == STET

/***************************************** MACROS and FUNCTIONS *************************************/
extern void InitStet();
extern void ReInitStet();
extern void MemorizeStetCorr(int PartInd, unsigned int NumCell[]);
extern double GetStetCorr(unsigned int NbPartInStet, unsigned int ix, unsigned int iy, unsigned int iz);
extern void CalcStetAddedNbPart();
extern double GetStetCorrOut(int PartInd, unsigned int NumCell[]);
extern void PrintStetInfo(FILE *pFile);
extern void TestStet();
#if GRAVIT_MODEL == STET
  #if STET_MODE == STET_SIMPLE 
    #define InsideStetDisk(ix, iy, iz, ajx, ajy, ajz)    (StetDisk[ajx][ajy][ajz])
  #else
    extern unsigned char InsideStetDisk( unsigned int ix, unsigned int iy, unsigned int iz, 
                                         unsigned int ax, unsigned int ay, unsigned int az);
  #endif
#else        // Newton case.
  #define InsideStetDisk(ix, iy, iz, ajx, ajy, ajz)  0
#endif
